@@ -55,11 +55,20 @@ foreach (array_keys($plugins) as $plugin) {
                 if ($plugin == 'blocktype') {
                     $key = blocktype_single_to_namespaced($i->name, $i->artefactplugin);
                 }
-                safe_require($plugin, $key);
+                try {
+                    safe_require($plugin, $key);
+                }
+                catch (SystemException $e) {
+                    $message = get_string('missingplugin', 'admin', hsc("$plugin:$key")) . ':<br>' . $e->getMessage();
+                    die_info($message);
+                }
                 $plugins[$plugin]['installed'][$key] = array(
                     'active' => $i->active,
                     'disableable' => call_static_method(generate_class_name($plugin, $key), 'can_be_disabled'),
                 );
+                if ($plugins[$plugin]['installed'][$key]['disableable']) {
+                    $plugins[$plugin]['installed'][$key]['activateform'] = activate_plugin_form($plugin, $i);
+                }
                 if ($plugin == 'artefact') {
                     $plugins[$plugin]['installed'][$key]['types'] = array();
                     safe_require('artefact', $key);
@@ -78,7 +87,7 @@ foreach (array_keys($plugins) as $plugin) {
                 } 
                 else {
                     $classname = generate_class_name($plugin, $i->name);
-                    safe_require($plugin, $i->name);
+                    safe_require($plugin, $key);
                     if (call_static_method($classname, 'has_config')) {
                         $plugins[$plugin]['installed'][$key]['config'] = true;
                     }
@@ -194,7 +203,5 @@ $smarty = smarty();
 $smarty->assign('INLINEJAVASCRIPT', $javascript);
 $smarty->assign('plugins', $plugins);
 $smarty->assign('installlink', 'installplugin');
-$smarty->assign('PAGEHEADING', hsc(TITLE));
+$smarty->assign('PAGEHEADING', TITLE);
 $smarty->display('admin/extensions/plugins.tpl');
-
-?>
