@@ -33,7 +33,8 @@ define('SECTION_PLUGINNAME', 'admin');
 define('SECTION_PAGE', 'sitemenu');
 
 require(dirname(dirname(dirname(__FILE__))) . '/init.php');
-define('TITLE', get_string('linksandresourcesmenu', 'admin'));
+require_once('pieforms/pieform.php');
+define('TITLE', get_string('menus', 'admin'));
 
 $strings = array('edit','delete','update','cancel','add','name','unknownerror');
 $adminstrings = array('confirmdeletemenuitem', 'deletefailed','deletingmenuitem','savingmenuitem',
@@ -87,17 +88,17 @@ function formatrow (item) {
     // item has id, type, name, link, linkedto
     var type = eval(item.type);
     var linkedto = A({'href':item.linkedto},item.linktext);
-    var del = INPUT({'type':'button','class':'button','value':{$getstring['delete']}});
-    connect(del, 'onclick', function () { delitem(item.id); });
     var edit = INPUT({'type':'button','class':'button','value':{$getstring['edit']}});
     connect(edit, 'onclick', function () { edititem(item); });
+    var del = INPUT({'type':'button','class':'button','value':{$getstring['delete']}});
+    connect(del, 'onclick', function () { delitem(item.id); });
     var cells = map(
         partial(TD,null),
         [
             type,
             item.name,
             linkedto,
-            [del,edit,contextualHelpIcon(null, null, 'core', 'admin', null, 'adminmenuedit')]
+            [edit,del,contextualHelpIcon(null, null, 'core', 'admin', null, 'adminmenuedit')]
         ]
     );
     return TR({'id':'menuitem_'+item.id},cells);
@@ -194,8 +195,8 @@ function closeopenedits() {
         if (hasElementClass(rows[i],'edit')) {
             removeElement(rows[i]);
         }
-        else if (hasElementClass(rows[i],'invisible')) {
-            removeElementClass(rows[i],'invisible');
+        else if (hasElementClass(rows[i],'hidden')) {
+            removeElementClass(rows[i],'hidden');
         }
     }
 }
@@ -216,7 +217,7 @@ function changeaddform(type) {
 function edititem(item) {
     closeopenedits();
     var menuitem = $('menuitem_'+item.id);
-    addElementClass(menuitem,'invisible');
+    addElementClass(menuitem,'hidden');
     var newrow = editform(item);
     insertSiblingNodesBefore(menuitem, newrow);
 }
@@ -273,12 +274,48 @@ foreach ($menulist as &$menu) {
                   'name' => get_string($menu,'admin'));
 }
 
-$style = '<style type="text/css">.invisible{display:none;}</style>';
-$smarty = smarty(array(), array($style));
+
+// Edit form for footer links
+$all = footer_menu(true);
+$active = get_config('footerlinks');
+$active = $active ? unserialize($active) : array_keys($all);
+$footerelements = array();
+foreach ($all as $k => $v) {
+    $footerelements[$k] = array(
+        'type' => 'checkbox',
+        'title' => $v['title'],
+        'defaultvalue' => in_array($k, $active),
+    );
+}
+$footerelements['submit'] = array(
+    'type'  => 'submit',
+    'value' => get_string('savechanges', 'admin')
+);
+$footerform = pieform(array(
+    'name'              => 'footerlinks',
+    'elements'          => $footerelements,
+));
+
+function footerlinks_submit(Pieform $form, $values) {
+    global $active, $all, $SESSION;
+    $new = array();
+    foreach (array_keys($all) as $k) {
+        if (!empty($values[$k])) {
+            $new[] = $k;
+        }
+    }
+    if ($new != $active) {
+        set_config('footerlinks', serialize($new));
+        $SESSION->add_ok_msg(get_string('footerupdated', 'admin'));
+    }
+    redirect(get_config('wwwroot') . 'admin/site/menu.php');
+}
+
+
+$smarty = smarty();
 $smarty->assign('INLINEJAVASCRIPT', $ijs);
 $smarty->assign('MENUS', $menulist);
 $smarty->assign('descriptionstrargs', array('<a href="' . get_config('wwwroot') . 'artefact/file/sitefiles.php">', '</a>'));
-$smarty->assign('PAGEHEADING', hsc(get_string('linksandresourcesmenu', 'admin')));
+$smarty->assign('PAGEHEADING', TITLE);
+$smarty->assign('footerform', $footerform);
 $smarty->display('admin/site/menu.tpl');
-
-?>
