@@ -29,6 +29,7 @@ define('ADMIN', 1);
 require(dirname(dirname(dirname(__FILE__))) . '/init.php');
 define('TITLE', get_string('institutions', 'admin'));
 require_once('pieforms/pieform.php');
+require_once(get_config('docroot') . '/lib/htmloutput.php');
 
 // CHECK FOR CANCEL BEFORE THE 'REQUIRED' PARAMS:
 $cancel = param_boolean('c');
@@ -149,23 +150,6 @@ function auth_config_submit(Pieform $form, $values) {
     exit;
 }
 
-// TODO: move to lib if people want this:
-function execute_javascript_and_close($js='') {
-    echo '<html>
-    <head>
-        <title>You may close this window</title>
-        <script language="Javascript">
-            function closeMe() { 
-                '.$js.'
-                window.close();
-            }
-        </script>
-    </head>
-    <body onLoad="closeMe();" style="font-family: Arial, Helvetica, sans-serif; font-size: 12px; text-align: center;">This window should close automatically</body>'.
-    "\n</html>";
-    exit;
-}
-
 $js = <<<EOF
 function authloginmsgVisibility() {
     // If Parent authority is 'None'
@@ -203,16 +187,17 @@ function updateSsoOptions() {
 }
 addLoadEvent(
     function() {
-        connect('auth_config_parent', 'onchange', authloginmsgVisibility);
-        authloginmsgVisibility();
+        if ($('auth_config_parent')) {
+            connect('auth_config_parent', 'onchange', authloginmsgVisibility);
+            authloginmsgVisibility();
+        }
         if ($('auth_config_ssodirection')) {
             connect('auth_config_ssodirection', 'onchange', updateSsoOptions);
+            updateSsoOptions();
         }
-        updateSsoOptions();
     }
 );
 EOF;
 
 $smarty->assign('INLINEJAVASCRIPT', $js);
 $smarty->display('admin/users/addauthority.tpl');
-?>
