@@ -38,6 +38,7 @@ require_once('activity.php');
 $id = param_integer('id');
 $user = new User;
 $user->find_by_id($id);
+$authobj = AuthFactory::create($user->authinstance);
 
 if (!$USER->is_admin_for_user($user)) {
     $SESSION->add_error_msg(get_string('youcannotadministerthisuser', 'admin'));
@@ -53,17 +54,34 @@ $elements['id'] = array(
     'rules'   => array('integer' => true),
     'value'   => $id,
 );
-$elements['password'] = array(
-    'type'         => 'text',
-    'title'        => get_string('resetpassword','admin'),
-    'description'  => get_string('resetpassworddescription','admin'),
-);
-$elements['passwordchange'] = array(
-    'type'         => 'checkbox',
-    'title'        => get_string('forcepasswordchange','admin'),
-    'description'  => get_string('forcepasswordchangedescription','admin'),
-    'defaultvalue' => $user->passwordchange,
-);
+
+if (method_exists($authobj, 'change_username')) {
+    $elements['username'] = array(
+        'type'         => 'text',
+        'title'        => get_string('changeusername', 'admin'),
+        'description'  => get_string('changeusernamedescription', 'admin'),
+        'defaultvalue' => $user->username,
+        'rules' => array(
+            'maxlength' => 236,
+         ),
+    );
+}
+
+if (method_exists($authobj, 'change_password')) {
+    // Only show the password options if the plugin allows for the functionality
+    $elements['password'] = array(
+        'type'         => 'text',
+        'title'        => get_string('resetpassword','admin'),
+        'description'  => get_string('resetpassworddescription','admin'),
+    );
+
+    $elements['passwordchange'] = array(
+        'type'         => 'checkbox',
+        'title'        => get_string('forcepasswordchange','admin'),
+        'description'  => get_string('forcepasswordchangedescription','admin'),
+        'defaultvalue' => $user->passwordchange,
+    );
+}
 if ($USER->get('admin')) {
     $elements['staff'] = array(
         'type'         => 'checkbox',
@@ -78,6 +96,12 @@ if ($USER->get('admin')) {
         'help'         => true,
     );
 }
+$elements['maildisabled'] = array(
+    'type' => 'checkbox',
+    'defaultvalue' => get_account_preference($user->id, 'maildisabled'),
+    'title' => get_string('email'),
+    'help' => true,
+);
 $elements['expiry'] = array(
     'type'         => 'date',
     'title'        => get_string('accountexpiry', 'admin'),
@@ -151,21 +175,79 @@ $siteform = pieform(array(
     'elements'   => $elements,
 ));
 
+function edituser_site_validate(Pieform $form, $values) {
+    global $USER, $SESSION;
+    if (!$user = get_record('usr', 'id', $values['id'])) {
+        return false;
+    }
+    $maxquotaenabled = get_config_plugin('artefact', 'file', 'maxquotaenabled');
+    $maxquota = get_config_plugin('artefact', 'file', 'maxquota');
+    if ($maxquotaenabled && $values['quota'] > $maxquota) {
+        $form->set_error('quota', get_string('maxquotaexceededform', 'artefact.file', display_size($maxquota)));
+        $SESSION->add_error_msg(get_string('maxquotaexceeded', 'artefact.file', display_size($maxquota)));
+    }
+
+    $userobj = new User();
+    $userobj = $userobj->find_by_id($user->id);
+
+    if (isset($values['username']) && !empty($values['username']) && $values['username'] != $userobj->username) {
+
+        if (!isset($values['authinstance'])) {
+            $authobj = AuthFactory::create($userobj->authinstance);
+        }
+        else {
+            $authobj = AuthFactory::create($values['authinstance']);
+        }
+
+        if (method_exists($authobj, 'change_username')) {
+
+            if (method_exists($authobj, 'is_username_valid_admin')) {
+                if (!$authobj->is_username_valid_admin($values['username'])) {
+                    $form->set_error('username', get_string('usernameinvalidadminform', 'auth.internal'));
+                }
+            }
+            else if (method_exists($authobj, 'is_username_valid')) {
+                if (!$authobj->is_username_valid($values['username'])) {
+                    $form->set_error('username', get_string('usernameinvalidform', 'auth.internal'));
+                }
+            }
+
+            if (!$form->get_error('username') && record_exists_select('usr', 'LOWER(username) = ?', strtolower($values['username']))) {
+                $form->set_error('username', get_string('usernamealreadytaken', 'auth.internal'));
+            }
+        }
+        else {
+            $form->set_error('username', get_string('usernamechangenotallowed', 'admin'));
+        }
+    }
+
+    // Check that the external username isn't already in use
+    if (isset($values['remoteusername']) &&
+        $usedby = get_record_select('auth_remote_user',
+        'authinstance = ? AND remoteusername = ? AND localusr != ?',
+        array($values['authinstance'], $values['remoteusername'], $values['id']))
+    ) {
+        $usedbyuser = get_field('usr', 'username', 'id', $usedby->localusr);
+        $SESSION->add_error_msg(get_string('duplicateremoteusername', 'auth', $usedbyuser));
+        $form->set_error('remoteusername', get_string('duplicateremoteusernameformerror', 'auth'));
+    }
+}
 
 function edituser_site_submit(Pieform $form, $values) {
+    global $USER, $authobj, $SESSION;
+
     if (!$user = get_record('usr', 'id', $values['id'])) {
         return false;
     }
 
-    if (isset($values['password']) && $values['password'] !== '') {
-        $user->password = $values['password'];
-        $user->salt = '';
-    }
-    $user->passwordchange = (int) ($values['passwordchange'] == 'on');
     $user->quota = $values['quota'];
     $user->expiry = db_format_timestamp($values['expiry']);
 
-    global $USER;
+
+    // Try to kick the user from any active login sessions, before saving data.
+    require_once(get_config('docroot') . 'auth/session.php');
+    remove_user_sessions($user->id);
+
     if ($USER->get('admin')) {  // Not editable by institutional admins
         $user->staff = (int) ($values['staff'] == 'on');
         $user->admin = (int) ($values['admin'] == 'on');
@@ -174,6 +256,17 @@ function edituser_site_submit(Pieform $form, $values) {
         }
     }
 
+    if ($values['maildisabled'] == 0 && get_account_preference($user->id, 'maildisabled') == 1) {
+        // Reset the sent and bounce counts otherwise mail will be disabled
+        // on the next send attempt
+        $u = new StdClass;
+        $u->email = $user->email;
+        $u->id = $user->id;
+        update_bounce_count($u,true);
+        update_send_count($u,true);
+    }
+    set_account_preference($user->id, 'maildisabled', $values['maildisabled']);
+
     // Authinstance can be changed by institutional admins if both the
     // old and new authinstances belong to the admin's institutions
     $remotename = get_field('auth_remote_user', 'remoteusername', 'authinstance', $user->authinstance, 'localusr', $user->id);
@@ -188,7 +281,7 @@ function edituser_site_submit(Pieform $form, $values) {
         if ($USER->get('admin') || 
             ($USER->is_institutional_admin($authinst[$values['authinstance']]->institution) &&
              $USER->is_institutional_admin($authinst[$user->authinstance]->institution))) {
-            delete_records('auth_remote_user', 'authinstance', $user->authinstance, 'localusr', $user->id);
+            delete_records('auth_remote_user', 'localusr', $user->id);
             if ($authinst[$values['authinstance']]->authname != 'internal') {
                 if (isset($values['remoteusername']) && strlen($values['remoteusername']) > 0) {
                     $un = $values['remoteusername'];
@@ -203,9 +296,65 @@ function edituser_site_submit(Pieform $form, $values) {
                 ));
             }
             $user->authinstance = $values['authinstance'];
+
+            // update the global $authobj to match the new authinstance
+            // this is used by the password/username change methods
+            // if either/both has been requested at the same time
+            $authobj = AuthFactory::create($user->authinstance);
         }
     }
 
+    // Only change the pw if the new auth instance allows for it
+    if (method_exists($authobj, 'change_password')) {
+        $user->passwordchange = (int) ($values['passwordchange'] == 'on');
+
+        if (isset($values['password']) && $values['password'] !== '') {
+            $userobj = new User();
+            $userobj = $userobj->find_by_id($user->id);
+
+            $user->password = $authobj->change_password($userobj, $values['password']);
+            $user->salt = $userobj->salt;
+
+            unset($userobj);
+        }
+    } else {
+        // inform the user that the chosen auth instance doesn't allow password changes
+        // but only if they tried changing it
+        if (isset($values['password']) && $values['password'] !== '') {
+            $SESSION->add_error_msg(get_string('passwordchangenotallowed', 'admin'));
+
+            // Set empty pw with salt
+            $user->password = '';
+            $user->salt = auth_get_random_salt();
+        }
+    }
+
+    if (isset($values['username']) && $values['username'] !== '') {
+        $userobj = new User();
+        $userobj = $userobj->find_by_id($user->id);
+
+        if ($userobj->username != $values['username']) {
+            // Only change the username if the auth instance allows for it
+            if (method_exists($authobj, 'change_username')) {
+                // check the existence of the chosen username
+                try {
+                    if ($authobj->user_exists($values['username'])) {
+                        // set an error message if it is already in use
+                        $SESSION->add_error_msg(get_string('usernameexists', 'account'));
+                    }
+                } catch (AuthUnknownUserException $e) {
+                    // update the username otherwise
+                    $user->username = $authobj->change_username($userobj, $values['username']);
+                }
+            } else {
+                // inform the user that the chosen auth instance doesn't allow username changes
+                $SESSION->add_error_msg(get_string('usernamechangenotallowed', 'admin'));
+            }
+        }
+
+        unset($userobj);
+    }
+
     update_record('usr', $user);
 
     redirect('/admin/users/edit.php?id='.$user->id);
@@ -227,7 +376,7 @@ if (empty($suspended)) {
             'reason' => array(
                 'type'        => 'textarea',
                 'rows'        => 5,
-                'cols'        => 60,
+                'cols'        => 28,
                 'title'       => get_string('reason'),
                 'description' => get_string('suspendedreasondescription', 'admin'),
             ),
@@ -330,7 +479,7 @@ $elements = array(
      ),
 );
 
-$allinstitutions = get_records_assoc('institution');
+$allinstitutions = get_records_assoc('institution', '', '', 'displayname');
 foreach ($user->get('institutions') as $i) {
     $elements[$i->institution.'_settings'] = array(
         'type' => 'fieldset',
@@ -461,7 +610,13 @@ function edituser_institution_submit(Pieform $form, $values) {
 
     if (isset($values['add']) && $USER->get('admin')
         && (empty($userinstitutions) || get_config('usersallowedmultipleinstitutions'))) {
-        $user->join_institution($values['addinstitution']);
+        if ($user->id == $USER->id) {
+            $USER->join_institution($values['addinstitution']);
+            $USER->commit();
+        }
+        else {
+            $user->join_institution($values['addinstitution']);
+        }
     }
 
     redirect('/admin/users/edit.php?id='.$user->id);
@@ -482,13 +637,15 @@ $smarty->assign('siteform', $siteform);
 $smarty->assign('institutions', count($allinstitutions) > 1);
 $smarty->assign('institutionform', $institutionform);
 
-if ($id != $USER->get('id') && is_null($USER->get('parentuser'))) {
-    $loginas = get_string('loginasuser', 'admin', hsc($user->username));
-} else {
-    $loginas = null;
-}
-$smarty->assign('loginas', $loginas);
-$smarty->assign('PAGEHEADING', hsc(TITLE . ': ' . display_name($user)));
-$smarty->display('admin/users/edit.tpl');
+$smarty->assign('loginas', $id != $USER->get('id') && is_null($USER->get('parentuser')));
+$smarty->assign('PAGEHEADING', TITLE . ': ' . display_name($user));
 
-?>
+# Only allow deletion and suspension of a user if the viewed user is not
+# the current user; or if they are the current user, they're not the only
+# admin
+if ($id != $USER->get('id') || count_records('usr', 'admin', 1, 'deleted', 0) > 1) {
+    $smarty->assign('suspendable', ($USER->get('admin') || !$user->get('admin') && !$user->get('staff')));
+    $smarty->assign('deletable', $USER->get('admin'));
+}
+
+$smarty->display('admin/users/edit.tpl');
