@@ -57,6 +57,11 @@ function pieform_element_filebrowser(Pieform $form, $element) {
 
     $userid = ($group || $institution) ? null : $USER->get('id');
 
+    // refresh quotas
+    if ($userid) {
+        $USER->quota_refresh();
+    }
+
     $folder = $element['folder'];
     $path = pieform_element_filebrowser_get_path($folder);
     $smarty->assign('folder', $folder);
@@ -102,9 +107,17 @@ function pieform_element_filebrowser(Pieform $form, $element) {
             $smarty->assign('agreementtext', get_string('uploadcopyrightdefaultcontent', 'install'));
         }
     }
+    else if (!isset($config['simpleupload'])) {
+        $config['simpleupload'] = 1;
+    }
+
     if ($config['upload']) {
-        $maxuploadsize = display_size(min(get_real_size(ini_get('post_max_size')), get_real_size(ini_get('upload_max_filesize'))));
+        $maxuploadsize = display_size(get_max_upload_size(!$institution && !$group));
         $smarty->assign('maxuploadsize', $maxuploadsize);
+        $smarty->assign('phpmaxfilesize', get_max_upload_size(false));
+        if ($group) {
+            $smarty->assign('uploaddisabled', !pieform_element_filebrowser_edit_group_folder($group, $folder));
+        }
     }
 
     if (!empty($element['browsehelp'])) {
@@ -152,7 +165,7 @@ function pieform_element_filebrowser_get_groupinfo($group) {
         'perm'  => array(),
     );
     foreach (current($groupinfo['perms']) as $k => $v) {
-        $groupinfo['perm'][$k] = get_string($k);
+        $groupinfo['perm'][$k] = get_string('filepermission.' . $k, 'artefact.file');
     }
     return $groupinfo;
 }
@@ -169,7 +182,7 @@ function pieform_element_filebrowser_get_path($folder) {
         }
     }
 
-    $path[] = (object) array('title' => get_string('home'), 'id' => 0);
+    $path[] = (object) array('title' => get_string('home', 'artefact.file'), 'id' => 0);
     return $path;
 }
 
@@ -201,10 +214,15 @@ function pieform_element_filebrowser_build_filelist($form, $element, $folder, $h
     global $USER;
 
     $smarty = smarty_core();
-
+    $userid = null;
     if (is_null($group) && is_null($user)) {
         $group = $form->get_property('group');
     }
+    else if ($user) {
+        $userid = $USER->get('id');
+        $smarty->assign('owner', 'user');
+        $smarty->assign('ownerid', $userid);
+    }
     else {
         $smarty->assign('owner', 'group');
         $smarty->assign('ownerid', $group);
@@ -216,7 +234,10 @@ function pieform_element_filebrowser_build_filelist($form, $element, $folder, $h
         $smarty->assign('owner', 'institution');
         $smarty->assign('ownerid', $institution);
     }
-    $userid = ($group || $institution) ? null : $USER->get('id');
+
+    if (is_null($userid)) {
+        $userid = ($group || $institution) ? null : $USER->get('id');
+    }
     $editable = (int) $element['config']['edit'];
     $selectable = (int) $element['config']['select'];
     $selectfolders = (int) !empty($element['config']['selectfolders']);
@@ -333,10 +354,10 @@ function pieform_element_filebrowser_get_value(Pieform $form, $element) {
             $replacehtml = false; // Don't replace the entire form when replying with json data.
             $result['formelement'] = $prefix;
             if (!empty($result['error'])) {
-                $result['formelementerror'] = $prefix . '.success';
+                $result['formelementerror'] = $prefix . '.callback';
             }
             else {
-                $result['formelementsuccess'] = $prefix . '.success';
+                $result['formelementsuccess'] = $prefix . '.callback';
             }
             $form->json_reply(empty($result['error']) ? PIEFORM_OK : PIEFORM_ERR, $result, $replacehtml);
         }
@@ -427,6 +448,7 @@ function pieform_element_filebrowser_doupdate(Pieform $form, $element) {
             'description' => param_variable($prefix . '_edit_description'),
             'tags'        => param_variable($prefix . '_edit_tags'),
             'folder'      => $element['folder'],
+            'allowcomments' => param_boolean($prefix . '_edit_allowcomments'),
         );
         if ($form->get_property('group')) {
             $data['permissions']  = array('admin' => (object) array('view' => true, 'edit' => true, 'republish' => true));
@@ -483,7 +505,7 @@ function pieform_element_filebrowser_doupdate(Pieform $form, $element) {
         }
     }
 
-    if (isset($_FILES['userfile']['error']) && $_FILES['userfile']['error'] == 0) {
+    if (!empty($_FILES['userfile']['name'])) {
         if (strlen($_FILES['userfile']['name']) > 1024) {
             return array(
                 'error'   => true,
@@ -523,7 +545,16 @@ function pieform_element_filebrowser_doupdate(Pieform $form, $element) {
             $keys = array_keys($select);
             $add = (int) $keys[0];
             if (isset($element['selectcallback']) && is_callable($element['selectcallback'])) {
-                $element['selectcallback']($add);
+                try {
+                    $element['selectcallback']($add);
+                }
+                catch (ArtefactNotFoundException $e) {
+                    $result = array(
+                        'error' => true,
+                        'message' => get_string('selectingfailed', 'artefact.file'),
+                    );
+                    return $result;
+                }
             }
             else {
                 $result['select'] = $add;
@@ -538,7 +569,16 @@ function pieform_element_filebrowser_doupdate(Pieform $form, $element) {
             $keys = array_keys($unselect);
             $del = (int) $keys[0];
             if (isset($element['unselectcallback']) && is_callable($element['unselectcallback'])) {
-                $element['unselectcallback']($del);
+                try {
+                    $element['unselectcallback']($del);
+                }
+                catch (ArtefactNotFoundException $e) {
+                    $result = array(
+                        'error' => true,
+                        'message' => get_string('removingfailed', 'artefact.file'),
+                    );
+                    return $result;
+                }
             }
             else {
                 $result['unselect'] = $del;
@@ -608,6 +648,11 @@ function pieform_element_filebrowser_upload(Pieform $form, $element, $data) {
             $result['message'] = get_string('cannoteditfolder', 'artefact.file');
             return $result;
         }
+        else if ($parentartefact->get('locked')) {
+            $result['error'] = true;
+            $result['message'] = get_string('cannoteditfoldersubmitted', 'artefact.file');
+            return $result;
+        }
         $parentfoldername = $parentartefact->get('title');
     }
     else {
@@ -621,21 +666,9 @@ function pieform_element_filebrowser_upload(Pieform $form, $element, $data) {
         }
         $data->institution = $institution;
     } else if ($group) {
-        require_once(get_config('libroot') . 'group.php');
         if (!$parentfolder) {
-            $role = group_user_access($group);
-            if (!$role) {
-                $result['error'] = true;
-                $result['message'] = get_string('usernotingroup', 'mahara');
-                return $result;
-            }
-            // Use default grouptype artefact permissions to check if the
-            // user can upload a file to the group's root directory
-            $permissions = group_get_default_artefact_permissions($group);
-            if (!$permissions[$role]->edit) {
-                $result['error'] = true;
-                $result['message'] = get_string('cannoteditfolder', 'artefact.file');
-                return $result;
+            if (!pieform_element_filebrowser_edit_group_folder($group, 0)) {
+                return array('error' => true, 'message' => get_string('cannoteditfolder', 'artefact.file'));
             }
         }
         $data->group = $group;
@@ -644,7 +677,6 @@ function pieform_element_filebrowser_upload(Pieform $form, $element, $data) {
     }
 
     $data->container = 0;
-    $data->locked = 0;
 
     $originalname = $_FILES['userfile']['name'];
     $originalname = $originalname ? basename($originalname) : get_string('file', 'artefact.file');
@@ -655,6 +687,10 @@ function pieform_element_filebrowser_upload(Pieform $form, $element, $data) {
     }
     catch (QuotaExceededException $e) {
         prepare_upload_failed_message($result, $e, $parentfoldername, $originalname);
+        // update the file listing
+        $result['quota'] = $USER->get('quota');
+        $result['quotaused'] = $USER->get('quotaused');
+        $result['newlist'] = pieform_element_filebrowser_build_filelist($form, $element, $parentfolder);
         return $result;
     }
     catch (UploadException $e) {
@@ -702,6 +738,7 @@ function pieform_element_filebrowser_upload(Pieform $form, $element, $data) {
     $result['newlist'] = pieform_element_filebrowser_build_filelist($form, $element, $parentfolder, $newid);
     $result['quota'] = $USER->get('quota');
     $result['quotaused'] = $USER->get('quotaused');
+    $result['maxuploadsize'] = display_size(get_max_upload_size(!$institution && !$group));
     return $result;
 }
 
@@ -737,22 +774,20 @@ function pieform_element_filebrowser_createfolder(Pieform $form, $element, $data
         'title'       => $data['title'],
     );
 
-    if ($parentfolder && !$USER->can_edit_artefact(artefact_instance_from_id($parentfolder))) {
-        return array('error' => true, 'message' => get_string('cannoteditfolder', 'artefact.file'));
+    if ($parentfolder) {
+        $parentartefact = artefact_instance_from_id($parentfolder);
+        if (!$USER->can_edit_artefact($parentartefact)) {
+            return array('error' => true, 'message' => get_string('cannoteditfolder', 'artefact.file'));
+        }
+        else if ($parentartefact->get('locked')) {
+            return array('error' => true, 'message' => get_string('cannoteditfoldersubmitted', 'artefact.file'));
+        }
     }
     if ($institution) {
         $data->institution = $institution;
     } else if ($group) {
-        require_once(get_config('libroot') . 'group.php');
         if (!$parentfolder) {
-            $role = group_user_access($group);
-            if (!$role) {
-                return array('error' => true, 'message' => get_string('usernotingroup', 'mahara'));
-            }
-            // Use default grouptype artefact permissions to check if the
-            // user can create a folder in the group's root directory
-            $permissions = group_get_default_artefact_permissions($group);
-            if (!$permissions[$role]->edit) {
+            if (!pieform_element_filebrowser_edit_group_folder($group, 0)) {
                 return array('error' => true, 'message' => get_string('cannoteditfolder', 'artefact.file'));
             }
         }
@@ -782,8 +817,20 @@ function pieform_element_filebrowser_update(Pieform $form, $element, $data) {
     global $USER;
     $collide = !empty($data['collide']) ? $data['collide'] : 'fail';
 
-    $artefact = artefact_instance_from_id($data['artefact']);
-    if (!$USER->can_edit_artefact($artefact)) {
+    try {
+        $artefact = artefact_instance_from_id($data['artefact']);
+    }
+    catch (ArtefactNotFoundException $e) {
+        $parentfolder = $element['folder'] ? $element['folder'] : null;
+        $result = array(
+            'error'   => true,
+            'message' => get_string('editingfailed', 'artefact.file'),
+            'newlist' => pieform_element_filebrowser_build_filelist($form, $element, $parentfolder),
+        );
+        return $result;
+    }
+
+    if (!$USER->can_edit_artefact($artefact) || $artefact->get('locked')) {
         return array('error' => true, 'message' => get_string('noeditpermission', 'mahara'));
     }
 
@@ -803,6 +850,7 @@ function pieform_element_filebrowser_update(Pieform $form, $element, $data) {
 
     $artefact->set('title', $data['title']);
     $artefact->set('description', $data['description']);
+    $artefact->set('allowcomments', (int) $data['allowcomments']);
 
     $oldtags = $artefact->get('tags');
     $newtags = preg_split("/\s*,\s*/", trim($data['tags']));
@@ -835,19 +883,35 @@ function pieform_element_filebrowser_update(Pieform $form, $element, $data) {
 
 function pieform_element_filebrowser_delete(Pieform $form, $element, $artefact) {
     global $USER;
-    $artefact = artefact_instance_from_id($artefact);
-    if (!$USER->can_edit_artefact($artefact)) {
+    $institution = $form->get_property('institution');
+    $group       = $form->get_property('group');
+
+    try {
+        $artefact = artefact_instance_from_id($artefact);
+    }
+    catch (ArtefactNotFoundException $e) {
+        $parentfolder = $element['folder'] ? $element['folder'] : null;
+        $result = array(
+            'error'   => true,
+            'message' => get_string('deletingfailed', 'artefact.file'),
+            'newlist' => pieform_element_filebrowser_build_filelist($form, $element, $parentfolder),
+        );
+        return $result;
+    }
+
+    if (!$USER->can_edit_artefact($artefact) || $artefact->get('locked')) {
         return array('error' => true, get_string('nodeletepermission', 'mahara'));
     }
     $parentfolder = $artefact->get('parent');
     $artefact->delete();
     return array(
-        'error' => false, 
-        'deleted' => true, 
-        'message' => get_string('filethingdeleted', 'artefact.file', 
-                                get_string($artefact->get('artefacttype'), 'artefact.file')),
+        'error' => false,
+        'deleted' => true,
+        'message' => get_string('filethingdeleted', 'artefact.file',
+                                get_string($artefact->get('artefacttype'), 'artefact.file') . ' ' . $artefact->get('title')),
         'quotaused' => $USER->get('quotaused'),
         'quota' => $USER->get('quota'),
+        'maxuploadsize' => display_size(get_max_upload_size(!$institution && !$group)),
         'newlist' => pieform_element_filebrowser_build_filelist($form, $element, $parentfolder),
     );
 }
@@ -858,7 +922,17 @@ function pieform_element_filebrowser_move(Pieform $form, $element, $data) {
     $artefactid  = $data['artefact'];    // Artefact being moved
     $newparentid = $data['newparent'];   // Folder to move it to
 
-    $artefact = artefact_instance_from_id($artefactid);
+    try {
+        $artefact = artefact_instance_from_id($artefactid);
+    }
+    catch (ArtefactNotFoundException $e) {
+        $result = array(
+            'error' => true,
+            'message' => get_string('movingfailed', 'artefact.file'),
+            'newlist' => pieform_element_filebrowser_build_filelist($form, $element, $data['folder']),
+        );
+        return $result;
+    }
 
     if (!$USER->can_edit_artefact($artefact)) {
         return array('error' => true, 'message' => get_string('movefailednotowner', 'artefact.file'));
@@ -874,10 +948,24 @@ function pieform_element_filebrowser_move(Pieform $form, $element, $data) {
         if ($newparentid == $artefact->get('parent')) {
             return array('error' => false, 'message' => get_string('filealreadyindestination', 'artefact.file'));
         }
-        $newparent = artefact_instance_from_id($newparentid);
+        try {
+            $newparent = artefact_instance_from_id($newparentid);
+        }
+        catch (ArtefactNotFoundException $e) {
+            $parentfolder = $element['folder'] ? $element['folder'] : null;
+            $result = array(
+                'error' => true,
+                'message' => get_string('movingfailed', 'artefact.file'),
+                'newlist' => pieform_element_filebrowser_build_filelist($form, $element, $data['folder']),
+            );
+            return $result;
+        }
         if (!$USER->can_edit_artefact($newparent)) {
             return array('error' => true, 'message' => get_string('movefailednotowner', 'artefact.file'));
         }
+        else if ($newparent->get('locked')) {
+            return array('error' => true, 'message' => get_string('cannoteditfoldersubmitted', 'artefact.file'));
+        }
         $group = $artefact->get('group');
         if ($group && $group !== $newparent->get('group')) {
             return array('error' => true, 'message' => get_string('movefailednotowner', 'artefact.file'));
@@ -900,17 +988,21 @@ function pieform_element_filebrowser_move(Pieform $form, $element, $data) {
         }
         $group = $artefact->get('group');
         if ($group) {
-            // Use default grouptype artefact permissions to check if the
-            // user can move a file to the group's root directory
-            require_once(get_config('libroot') . 'group.php');
-            $permissions = group_get_default_artefact_permissions($group);
-            if (!$permissions[group_user_access($group)]->edit) {
+            if (!pieform_element_filebrowser_edit_group_folder($group, 0)) {
                 return array('error' => true, 'message' => get_string('movefailednotowner', 'artefact.file'));
             }
         }
         $newparentid = null;
     }
 
+    if ($oldparentid = $artefact->get('parent')) {
+        $oldparent = artefact_instance_from_id($oldparentid);
+        if ($oldparent->get('locked')) {
+            return array('error' => true, 'message' => get_string('cannotremovefromsubmittedfolder', 'artefact.file'));
+        }
+    }
+
+
     if ($artefact->move($newparentid)) {
         return array(
             'error' => false,
@@ -921,6 +1013,24 @@ function pieform_element_filebrowser_move(Pieform $form, $element, $data) {
 }
 
 
+function pieform_element_filebrowser_edit_group_folder($group, $folder) {
+    global $USER;
+    if ($folder) {
+        if (!$folder instanceof ArtefactTypeFolder) {
+            $folder = new ArtefactTypeFolder($folder);
+        }
+        return $USER->can_edit_artefact($folder);
+    }
+    require_once(get_config('libroot') . 'group.php');
+    // Group root directory: use default grouptype artefact permissions
+    if (!$role = group_user_access($group)) {
+        return false;
+    }
+    $permissions = group_get_default_artefact_permissions($group);
+    return $permissions[$role]->edit;
+}
+
+
 function pieform_element_filebrowser_changeowner(Pieform $form, $element) {
     $newtabdata = pieform_element_filebrowser_configure_tabs($element['tabs']);
     $smarty = smarty_core();
@@ -957,6 +1067,7 @@ function pieform_element_filebrowser_changeowner(Pieform $form, $element) {
         'changedfolder' => true,
         'tabupload'     => $newtabdata['upload'],
         'folder'        => $folder,
+        'disableedit'   => $group && !pieform_element_filebrowser_edit_group_folder($group, $folder),
         'newlist'       => pieform_element_filebrowser_build_filelist($form, $element, $folder, null, $user, $group, $institution),
         'newpath'       => pieform_element_filebrowser_build_path($form, $element, $folder, $newtabdata['owner'], $newtabdata['ownerid']),
         'newtabs'       => $newtabhtml,
@@ -987,10 +1098,16 @@ function pieform_element_filebrowser_changefolder(Pieform $form, $element, $fold
         }
     }
     
+    // If changing to a group folder, check whether the user can edit it
+    if ($g = ($owner ? $group : $form->get_property('group'))) {
+        $editgroupfolder = pieform_element_filebrowser_edit_group_folder($g, $folder);
+    }
+
     return array(
         'error'         => false, 
         'changedfolder' => true,
         'folder'        => $folder,
+        'disableedit'   => isset($editgroupfolder) && $editgroupfolder == false,
         'newlist'       => pieform_element_filebrowser_build_filelist($form, $element, $folder, null, $user, $group, $institution),
         'newpath'       => pieform_element_filebrowser_build_path($form, $element, $folder, $owner, $ownerid),
     );
@@ -1039,5 +1156,3 @@ function pieform_element_filebrowser_set_attributes($element) {/*{{{*/
     $element['needsmultipart'] = true;
     return $element;
 }/*}}}*/
-
-?>
