@@ -49,9 +49,7 @@ class PluginBlocktypeResumefield extends PluginBlocktype {
         $configdata = $bi->get('configdata');
 
         if (!empty($configdata['artefactid'])) {
-            require_once(get_config('docroot') . 'artefact/lib.php');
-            $resumefield = artefact_instance_from_id($configdata['artefactid']);
-            return $resumefield->get('title');
+            return $bi->get_artefact_instance($configdata['artefactid'])->get('title');
         }
         return '';
     }
@@ -196,7 +194,7 @@ class PluginBlocktypeResumefield extends PluginBlocktype {
             if (!empty($impcfg['artefacttype'])) {
                 if ($artefactid = get_field_sql("SELECT id
                     FROM {artefact}
-                    WHERE owner = ?
+                    WHERE \"owner\" = ?
                     AND artefacttype = ?
                     AND artefacttype IN (
                         SELECT name
@@ -219,5 +217,3 @@ class PluginBlocktypeResumefield extends PluginBlocktype {
     }
 
 }
-
-?>
