@@ -28,7 +28,7 @@
 defined('INTERNAL') || die();
 
 /**
-* LEAP export plugin.  See http://wiki.cetis.ac.uk/LEAP2A_specification and
+* LEAP export plugin.  See http://wiki.cetis.ac.uk/2009-03/Leap2A_specification and
 * http://wiki.mahara.org/Developer_Area/Import%2f%2fExport/LEAP_Export
 */
 class PluginExportLeap extends PluginExport {
@@ -125,7 +125,9 @@ class PluginExportLeap extends PluginExport {
     public function export() {
         // the xml stuff
         $this->export_header();
+        $this->setup_links();
         $this->notify_progress_callback(10, get_string('exportingviews', 'export'));
+        $this->export_collections();
         $this->export_views();
         $this->notify_progress_callback(50, get_string('exportingartefacts', 'export'));
         $this->export_artefacts();
@@ -202,6 +204,44 @@ class PluginExportLeap extends PluginExport {
         $this->xml .= $this->smarty->fetch('export:leap:header.tpl');
     }
 
+
+    /**
+     * Export the collections
+     */
+    private function export_collections() {
+        foreach ($this->collections as $id => $collection) {
+            $this->smarty->assign('title',       $collection->get('name'));
+            $this->smarty->assign('id',          'portfolio:collection' . $id);
+            $this->smarty->assign('updated',     self::format_rfc3339_date(strtotime($collection->get('mtime'))));
+            $this->smarty->assign('created',     self::format_rfc3339_date(strtotime($collection->get('ctime'))));
+            $this->smarty->assign('summarytype', 'text');
+            $this->smarty->assign('summary',     $collection->get('description'));
+            $this->smarty->assign('contenttype', 'text');
+            $this->smarty->assign('content',     $collection->get('description'));
+            $this->smarty->assign('leaptype',    'selection');
+
+            $this->smarty->assign('categories', array(
+                array(
+                    'scheme' => 'selection_type',
+                    'term' => 'Website',
+                )
+            ));
+
+            $links = array();
+            if (!empty($this->links->collectionview[$id])) {
+                foreach (array_keys($this->links->collectionview[$id]) as $viewid) {
+                    $links[] = (object)array(
+                        'type' => 'has_part',
+                        'id'   => 'portfolio:view' . $viewid,
+                    );
+                }
+            }
+            $this->smarty->assign('links', $links);
+            $this->xml .= $this->smarty->fetch("export:leap:entry.tpl");
+        }
+    }
+
+
     /**
      * Export the views
      */
@@ -231,7 +271,9 @@ class PluginExportLeap extends PluginExport {
                 $this->smarty->assign('summary',     $content);
             }
             $this->smarty->assign('contenttype', 'xhtml');
-            $this->smarty->assign('content',     self::parse_xhtmlish_content($view->build_columns()));
+            if ($viewcontent = self::parse_xhtmlish_content($view->build_columns(), $view->get('id'))) {
+                $this->smarty->assign('content', $viewcontent);
+            }
             $this->smarty->assign('viewdata',    $config['columns']);
             $this->smarty->assign('layout',      $view->get_layout()->widths);
             $this->smarty->assign('type',        $config['type']);
@@ -269,6 +311,21 @@ class PluginExportLeap extends PluginExport {
         $viewlist = join(',', array_keys($this->views));
         $artefactlist = join(',', array_keys($this->artefacts));
 
+        // Views in collections
+        if ($this->collections) {
+            $collectionlist = join(',', array_keys($this->collections));
+            $records = get_records_select_array(
+                'collection_view',
+                "view IN ($viewlist) AND collection IN ($collectionlist)"
+            );
+            if ($records) {
+                foreach ($records as &$r) {
+                    $this->links->collectionview[$r->collection][$r->view] = 1;
+                    $this->links->viewcollection[$r->view][$r->collection] = 1;
+                }
+            }
+        }
+
         // Artefacts directly in views
         $records = get_records_select_array(
             'view_artefact',
@@ -403,33 +460,37 @@ class PluginExportLeap extends PluginExport {
     }
 
     private function get_links_for_view($viewid) {
-        static $viewartefactdata = null;
-        static $vaextra = null;
-        if (is_null($viewartefactdata)) {
-            $viewartefactdata = get_records_select_array('view_artefact', 'view IN (' . join(', ', array_keys($this->views)) . ')');
-        }
-        if (is_null($vaextra)) {
-            $vaextra = $this->get_view_extra_artefacts(true);
-        }
-
         $links = array();
-        foreach ($viewartefactdata as $va) {
-            if ($va->view == $viewid) {
+
+        if (!empty($this->links->viewcollection[$viewid])) {
+            foreach (array_keys($this->links->viewcollection[$viewid]) as $collectionid) {
                 $links[] = (object)array(
-                    'type' => 'leap2:has_part',
-                    'id'   => 'portfolio:artefact' . $va->artefact,
+                    'type' => 'is_part_of',
+                    'id'   => 'portfolio:collection' . $collectionid,
                 );
             }
         }
 
-        if (isset($vaextra[$viewid])) {
-            foreach ($vaextra[$viewid] as $artefactid) {
+        if (!empty($this->links->viewcontents[$viewid])) {
+            foreach (array_keys($this->links->viewcontents[$viewid]) as $artefactid) {
                 $links[] = (object)array(
-                    'type' => 'is_evidence_of', // Fix this
+                    'type' => 'leap2:has_part',
                     'id'   => 'portfolio:artefact' . $artefactid,
                 );
             }
         }
+
+        if (!empty($this->links->viewartefact[$viewid])) {
+            foreach ($this->links->viewartefact[$viewid] as $artefactid => $linktypes) {
+                foreach ($linktypes as $linktype) {
+                    $links[] = (object)array(
+                        'type' => $linktype,
+                        'id'   => 'portfolio:artefact' . $artefactid,
+                    );
+                }
+            }
+        }
+
         return $links;
     }
 
@@ -502,11 +563,8 @@ class PluginExportLeap extends PluginExport {
     * @return filename string use this to pass to add_enclosure_link
     */
     public function add_attachment($filepath, $newname) {
-        if (!file_exists($filepath)) {
-            throw new ParamOutOfRangeException("Tried to add non existant file $filepath");
-        }
-        if (empty($newname)) {
-            throw new ParamOutOfRangeException("Tried to add non existant file $filepath");
+        if (!file_exists($filepath) || empty($newname)) {
+            throw new ParamOutOfRangeException(get_string('nonexistentfile', 'export', $newname));
         }
         $newname = substr(str_replace('/', '_', $newname), 0, 245);
         $this->attachments[] = (object)array('file' => $filepath, 'name' => $newname);
@@ -532,7 +590,7 @@ class PluginExportLeap extends PluginExport {
      *
      * @return xhtml content or false for unmodified text content
      */
-    public static function parse_xhtmlish_content($content) {
+    public static function parse_xhtmlish_content($content, $viewid=null) {
         $dom = new DomDocument();
         $topel = $dom->createElement('tmp');
         $tmp = new DomDocument();
@@ -554,10 +612,15 @@ class PluginExportLeap extends PluginExport {
         } else if (@$tmp->loadHTML('<?xml encoding="UTF-8"><div>' . $content . '</div>')) {
             $xpath = new DOMXpath($tmp);
             $elements = $xpath->query('/html/body/div');
-            if ($elements->length == 1) {
-                $ourelement = $elements->item(0);
+            if ($elements->length != 1) {
+                if ($viewid) {
+                    log_warn("Leap2a export: invalid html found in view $viewid");
+                }
+                if ($elements->length < 1) {
+                    return false;
+                }
             }
-
+            $ourelement = $elements->item(0);
             $content = $dom->importNode($ourelement, true);
             $content->setAttribute('xmlns', 'http://www.w3.org/1999/xhtml');
             $topel->appendChild($content);
@@ -648,6 +711,7 @@ class LeapExportElement {
         }
         $this->smarty->assign('leaptype', $this->get_leap_type());
         $this->smarty->assign('author', $this->get_entry_author());
+        $this->smarty->assign('dates', $this->get_dates());
 
         if ($tags = $this->artefact->get('tags')) {
             $tags = array_map(create_function('$a',
@@ -671,9 +735,12 @@ class LeapExportElement {
     *
     * @param View $view to link to
     */
-    public function add_view_link($viewid) {
+    public function add_view_link($viewid, $rel=null) {
+        if (is_null($rel)) {
+            $rel = $this->get_view_relationship($viewid);
+        }
         if (array_key_exists($viewid, $this->exporter->get('views'))) {
-            $this->add_generic_link('view' . $viewid, $this->get_view_relationship($viewid));
+            $this->add_generic_link('view' . $viewid, $rel);
         }
     }
 
@@ -721,19 +788,39 @@ class LeapExportElement {
     * The resulting array is keyed on the LEAP portfolio:id (eg portfolio:artefact2)
     */
     public function add_links() {
-        if ($views = $this->artefact->get_views_metadata()) {
+        $id = $this->artefact->get('id');
+        if ($views = $this->exporter->artefact_in_view_links($id)) {
             foreach ($views as $view) {
-                $this->add_view_link($view->view);
+                $this->add_view_link($view);
             }
         }
-        if ($parent = $this->artefact->get_parent_instance()) {
+        if ($parent = $this->exporter->artefact_parent_link($id)) {
             $this->add_artefact_link($parent, $this->get_parent_relationship($parent));
         }
-        if ($children = $this->artefact->get_children_instances()) {
+        if ($children = $this->exporter->artefact_child_links($id)) {
             foreach ($children as $child) {
                 $this->add_artefact_link($child, $this->get_child_relationship($child));
             }
         }
+        if ($attachments = $this->exporter->artefact_attachment_links($id)) {
+            foreach ($attachments as $a) {
+                $this->add_artefact_link($a, 'related');
+            }
+        }
+        if ($views = $this->exporter->artefact_view_links($id)) {
+            foreach ($views as $viewid => $linktypes) {
+                foreach ($linktypes as $linktype) {
+                    $this->add_view_link($viewid, $linktype);
+                }
+            }
+        }
+        if ($artefacts = $this->exporter->artefact_artefact_links($id)) {
+            foreach ($artefacts as $artefactid => $linktypes) {
+                foreach ($linktypes as $linktype) {
+                    $this->add_artefact_link($this->exporter->artefacts[$artefactid], $linktype);
+                }
+            }
+        }
     }
 
     /**
@@ -743,12 +830,15 @@ class LeapExportElement {
      * {@link add_attachment} method on the exporter object.
      *
      * @param string $filename the relative path of the file (NOT including the filesdir)
+     * @param string $mimetype the (optional) mimetype of the file (according to atom
+     *                          spec the type attribute on an enclosure is optional)
      */
-    public function add_enclosure_link($filename) {
+    public function add_enclosure_link($filename, $mimetype = '') {
         $this->links[$filename] = (object)array(
             'id' => $this->exporter->get('filedir') . $filename,
             'type' => 'enclosure',
-            'file' => true
+            'file' => true,
+            'mimetype' => $mimetype
         );
     }
 
@@ -766,7 +856,7 @@ class LeapExportElement {
 
     /**
     * The LEAP element type
-    * See http://wiki.cetis.ac.uk/2009-03/LEAP2A_types
+    * See http://wiki.cetis.ac.uk/2009-03/Leap2A_types
     *
     * @return string
     */
@@ -809,13 +899,27 @@ class LeapExportElement {
     }
 
     /**
-    * The id of the entry's author
-    * Override this if the author is different from the portfolio holder
+    * The name of the entry's author
     *
-    * @return int
+    * @return string
     */
     public function get_entry_author() {
-        return;
+        if ($author = $this->artefact->get('author')) {
+            if ($author != $this->artefact->get('owner')) {
+                return display_name($author);
+            }
+            return;
+        }
+        return $this->artefact->get('authorname');
+    }
+
+    /**
+    * Get leap:date items for the entry
+    *
+    * @return array
+    */
+    public function get_dates() {
+        return array();
     }
 
     /**
@@ -894,7 +998,7 @@ class LeapExportElement {
 
     /**
      * Converts a tag to a 'normalised' tag, as per 
-     * http://wiki.cetis.ac.uk/2009-03/LEAP2A_categories#Plain_tags
+     * http://wiki.cetis.ac.uk/2009-03/Leap2A_categories#Plain_tags
      *
      * The method of normalisation isn't specified at
      * the time of this being written.
@@ -1023,5 +1127,3 @@ class LeapExportOutputFilter {
     }
 
 }
-
-?>
