@@ -100,9 +100,15 @@ function requestjoingroup_submit(Pieform $form, $values) {
             'users'   => array($groupadmin),
             'subject' => get_string_from_language($adminlang, 'grouprequestsubject', 'group'),
             'message' => $message,
-            'url'     => get_config('wwwroot') . 'group/members.php?id=' . $group->id . '&membershiptype=request'));
+            'url'     => get_config('wwwroot') . 'group/members.php?id=' . $group->id . '&membershiptype=request',
+            'strings' => (object) array(
+                'urltext' => (object) array(
+                    'key'     => 'pendingmembers',
+                    'section' => 'group',
+                ),
+            ),
+        ));
     }
     $SESSION->add_ok_msg(get_string('grouprequestsent', 'group'));
     redirect($goto);
 }
-?>
