@@ -27,15 +27,17 @@
 
 defined('INTERNAL') || die();
 
-$string['changepassworddesc'] = 'If you wish to change your password, please enter the details here';
+$string['changepassworddesc'] = 'New password';
 $string['changepasswordotherinterface'] = 'You may <a href="%s">change your password</a> through a different interface';
 $string['oldpasswordincorrect'] = 'This is not your current password';
 
 $string['changeusernameheading'] = 'Change username';
 $string['changeusername'] = 'New username';
-$string['changeusernamedesc'] = 'The username you use to log into %s.  Usernames are 3-30 characters long, and may contain letters, numbers, and most common symbols excuding spaces.';
+$string['changeusernamedesc'] = 'The username you use to log into %s.  Usernames are 3-30 characters long, and may contain letters, numbers, and most common symbols excluding spaces.';
 
-$string['accountoptionsdesc'] = 'You can set general account options here';
+$string['usernameexists'] = 'This username is taken, please choose another.';
+
+$string['accountoptionsdesc'] = 'General account options';
 $string['friendsnobody'] = 'Nobody may add me as a friend';
 $string['friendsauth'] = 'New friends require my authorisation';
 $string['friendsauto'] = 'New friends are automatically authorised';
@@ -45,6 +47,8 @@ $string['updatedfriendcontrolsetting'] = 'Updated friends control';
 $string['wysiwygdescr'] = 'HTML editor';
 $string['on'] = 'On';
 $string['off'] = 'Off';
+$string['disabled'] = 'Disabled';
+$string['enabled'] = 'Enabled';
 
 $string['messagesdescr'] = 'Messages from other users';
 $string['messagesnobody'] = 'Do not allow anyone to send me messages';
@@ -53,15 +57,33 @@ $string['messagesallow'] = 'Allow anyone to send me messages';
 
 $string['language'] = 'Language';
 
-$string['showviewcolumns'] = 'Show controls to add and remove columns when editing a view';
+$string['showviewcolumns'] = 'Show controls to add and remove columns when editing a page';
 
 $string['tagssideblockmaxtags'] = 'Maximum tags in cloud';
 $string['tagssideblockmaxtagsdescription'] = 'Maximum number of tags to display in your Tag Cloud';
 
+$string['enablemultipleblogs'] = 'Enable multiple journals';
+$string['enablemultipleblogsdescription']  = 'By default, you have one journal. If you would like to keep more than one journal, check this option.';
+$string['disablemultipleblogserror'] = 'You cannot disable multiple journals unless you only have one journal';
+
+$string['hiderealname'] = 'Hide real name';
+$string['hiderealnamedescription'] = 'Check this box if you have set a display name and you do not want other users to be able to find you by your real name in user searches.';
+
+$string['showhomeinfo'] = 'Show information about Mahara on the home page';
+
+$string['mobileuploadtoken'] = 'Mobile upload token';
+$string['mobileuploadtokendescription'] = 'Enter a token here and on your phone to enable uploads (note: it will change automatically after each upload. <br/>If you have any problems - simply reset it here and on your phone.';
+
 $string['prefssaved']  = 'Preferences saved';
 $string['prefsnotsaved'] = 'Failed to save your Preferences!';
 
+$string['maildisabled'] = 'E-mail disabled';
+$string['maildisabledbounce'] =<<< EOF
+Sending of e-mail to your e-mail address has been disabled as too many messages have been returned to the server.
+Please check that your e-mail account is working as expected before you re-enable e-mail on the account preferences at %s.
+EOF;
+$string['maildisableddescription'] = 'Sending of email to your account has been disabled. You may <a href="%s">re-enable your email</a> from the account preferences page.';
+
 $string['deleteaccount']  = 'Delete Account';
-$string['deleteaccountdescription']  = 'If you delete your account, your profile information and your Views will no longer be visible to other users.  The content of any forum posts you have written will still be visible, but the author\'s name will no longer be displayed.';
+$string['deleteaccountdescription']  = 'If you delete your account, your profile information and your pages will no longer be visible to other users.  The content of any forum posts you have written will still be visible, but the author\'s name will no longer be displayed.';
 $string['accountdeleted']  = 'Your account has been deleted.';
-?>
