@@ -326,8 +326,11 @@ function file_mime_type($file) {
         // if /usr/share/misc/magic is a directory then finfo still succeeds and 
         // doesn't fall back onto the .mcg magic_file
         // force /usr/share/misc/magic.mgc instead in this case
-
-        $magicfile = !is_file('/usr/share/misc/magic') ? '/usr/share/misc/magic.mgc' : null;
+        $MAGICPATH = '/usr/share/misc/magic';
+        $magicfile = null;
+        if (is_dir($MAGICPATH) or is_link($MAGICPATH)) {
+            $magicfile = '/usr/share/misc/magic.mgc';
+        }
 
         if (defined('FILEINFO_MIME_TYPE')) {
             if ($finfo = @new finfo(FILEINFO_MIME_TYPE, $magicfile)) {
@@ -473,6 +476,7 @@ function is_image_file($path) {
  *                if an appropriate file could not be located or generated
  */
 function get_dataroot_image_path($path, $id, $size=null) {
+    global $THEME;
     $dataroot = get_config('dataroot');
     $imagepath = $dataroot . $path;
     if (substr($imagepath, -1) == '/') {
@@ -520,6 +524,24 @@ function get_dataroot_image_path($path, $id, $size=null) {
 
             $imageinfo = getimagesize($originalimage);
             $originalmimetype = $imageinfo['mime'];
+
+            // gd can eat a lot of memory shrinking large images, so use a placeholder image
+            // here if necessary
+            if (isset($imageinfo[0]) && isset($imageinfo[1]) && isset($imageinfo['bits'])) {
+                $approxmem = $imageinfo[0] * $imageinfo[1] * ($imageinfo['bits'] / 8)
+                    * (isset($imageinfo['channels']) ? $imageinfo['channels'] : 3);
+            }
+            if (empty($approxmem) || $approxmem > get_config('maximageresizememory')) {
+                log_debug("Refusing to resize large image $originalimage $originalmimetype "
+                    . $imageinfo[0] . 'x' .  $imageinfo[1] . ' ' . $imageinfo['bits'] . '-bit');
+                $originalimage = $THEME->get_path('images/no_thumbnail.png');
+                if (empty($originalimage) || !is_readable($originalimage)) {
+                    return false;
+                }
+                $imageinfo = getimagesize($originalimage);
+                $originalmimetype = $imageinfo['mime'];
+            }
+
             $format = 'png';
             switch ($originalmimetype) {
                 case 'image/jpeg':
