@@ -40,7 +40,7 @@ defined('INTERNAL') || die();
  * @return mixed          The role the user has in the group, or false if they 
  *                        have no role in the group
  */
-function group_user_access($groupid, $userid=null) {
+function group_user_access($groupid, $userid=null, $refresh=null) {
     static $result;
 
     if (!is_logged_in()) {
@@ -50,7 +50,7 @@ function group_user_access($groupid, $userid=null) {
     $groupid = group_param_groupid($groupid);
     $userid  = group_param_userid($userid);
 
-    if (isset($result[$groupid][$userid])) {
+    if (isset($result[$groupid][$userid]) && !isset($refresh)) {
         return $result[$groupid][$userid];
     }
 
@@ -109,10 +109,8 @@ function group_can_change_role($groupid, $userid, $role) {
     // admin role permissions check
     if ($role == 'admin') {
         $group = group_current_group();
-        $user = new User();
-        $user->find_by_id($userid);
         safe_require('grouptype', $group->grouptype);
-        return in_array($group->jointype, call_static_method('GroupType' . $group->grouptype, 'user_allowed_join_types', $user));
+        return call_static_method('GroupType' . $group->grouptype, 'can_become_admin', $userid);
     }
 
     return true;
@@ -145,6 +143,12 @@ function group_change_role($groupid, $userid, $role) {
  * @returns boolean
  */
 function group_user_can_edit_views($groupid, $userid=null) {
+    // root user can always do whatever it wants
+    $sysuser = get_record('usr', 'username', 'root');
+    if ($sysuser->id == $userid) {
+        return true;
+    }
+
     if (!is_logged_in()) {
         return false;
     }
@@ -259,6 +263,7 @@ function group_create($data) {
             'name'           => $data['name'],
             'description'    => $data['description'],
             'grouptype'      => $data['grouptype'],
+            'category'       => $data['category'],
             'jointype'       => $data['jointype'],
             'ctime'          => $data['ctime'],
             'mtime'          => $data['ctime'],
@@ -307,6 +312,21 @@ function group_create($data) {
     }
 
     $data['id'] = $id;
+    // install the homepage
+    if ($t = get_record('view', 'type', 'grouphomepage', 'template', 1, 'owner', 0)) {
+        require_once('view.php');
+        $template = new View($t->id, (array)$t);
+        list($homepage) = View::create_from_template(array(
+            'group' => $id,
+            'title' => $template->get('title'),
+            'description' => $template->get('description'),
+            'type' => 'grouphomepage',
+        ), $t->id, 0, false);
+    }
+    insert_record('view_access', (object) array(
+        'view' => $homepage->get('id'),
+        'accesstype' => $data['public'] ? 'public' : 'loggedin',
+    ));
     handle_event('creategroup', $data);
     db_commit();
 
@@ -407,8 +427,6 @@ function group_user_can_leave($group, $userid=null) {
 /**
  * Removes a user from a group.
  *
- * Also removes view access given by the user to the group
- *
  * @param int $groupid ID of group
  * @param int $userid  ID of user to remove
  */
@@ -417,19 +435,7 @@ function group_remove_user($groupid, $userid=null, $force=false) {
     if (!$force && !group_user_can_leave($groupid, $userid)) {
         throw new AccessDeniedException(get_string('usercantleavegroup', 'group'));
     }
-    db_begin();
     delete_records('group_member', 'group', $groupid, 'member', $userid);
-    delete_records_sql(
-        'DELETE FROM {view_access_group}
-        WHERE "group" = ?
-        AND view IN (
-            SELECT v.id
-            FROM {view} v
-            WHERE v.owner = ?
-        )',
-        array($groupid, $userid)
-    );
-    db_commit();
 
     require_once(get_config('docroot') . 'interaction/lib.php');
     $interactions = get_column('interaction_instance', 'id', 'group', $groupid);
@@ -438,6 +444,34 @@ function group_remove_user($groupid, $userid=null, $force=false) {
     }
 }
 
+/**
+ * Invite a user to a group.
+ *
+ * @param object $group group
+ * @param object $userid  User to invite
+ * @param object $userfrom  User sending the invitation
+ */
+function group_invite_user($group, $userid, $userfrom, $role='member', $delay=null) {
+    $user = optional_userobj($userid);
+
+    $data = new StdClass;
+    $data->group = $group->id;
+    $data->member= $user->id;
+    $data->ctime = db_format_timestamp(time());
+    $data->role = $role;
+    ensure_record_exists('group_member_invite', $data, $data);
+    $lang = get_user_language($user->id);
+    require_once('activity.php');
+    $activitydata = array(
+        'users'   => array($user->id),
+        'subject' => get_string_from_language($lang, 'invitetogroupsubject', 'group'),
+        'message' => get_string_from_language($lang, 'invitetogroupmessage', 'group', display_name($userfrom, $user), $group->name),
+        'url'     => get_config('wwwroot') . 'group/view.php?id=' . $group->id,
+        'urltext' => $group->name,
+    );
+    activity_occurred('maharamessage', $activitydata, null, null, $delay);
+}
+
 // Pieforms for various operations on groups
 
 /**
@@ -546,6 +580,32 @@ function group_get_removeuser_form($userid, $groupid) {
     ));
 }
 
+/**
+ * Form for denying request (request jointype group)
+ */
+function group_get_denyuser_form($userid, $groupid) {
+    require_once('pieforms/pieform.php');
+    return pieform(array(
+        'name'                => 'denyuser' . $userid,
+        'successcallback'     => 'group_denyuser_submit',
+        'renderer'            => 'oneline',
+        'elements'            => array(
+            'group' => array(
+                'type'    => 'hidden',
+                'value' => $groupid,
+            ),
+            'member' => array(
+                'type'  => 'hidden',
+                'value' => $userid,
+            ),
+            'denyuser' => array(
+                'type'  => 'submit',
+                'value' => get_string('declinerequest', 'group'),
+            ),
+        ),
+    ));
+}
+
 // Functions for handling submission of group related forms
 
 function joingroup_submit(Pieform $form, $values) {
@@ -599,6 +659,24 @@ function group_adduser_submit(Pieform $form, $values) {
     redirect('/group/members.php?id=' . $group);
 }
 
+/**
+ * Denying request (request jointype group)
+ */
+function group_denyuser_submit(Pieform $form, $values) {
+    global $SESSION;
+    $group = (int)$values['group'];
+    if (group_user_access($group) != 'admin') {
+        $SESSION->add_error_msg(get_string('accessdenied', 'error'));
+        redirect('/group/members.php?id=' . $group . '&membershiptype=request');
+    }
+    delete_records('group_member_request', 'group', $values['group'], 'member', $values['member']);
+    $SESSION->add_ok_msg(get_string('declinerequestsuccess', 'group'));
+    if (count_records('group_member_request', 'group', $group)) {
+        redirect('/group/members.php?id=' . $group . '&membershiptype=request');
+    }
+    redirect('/group/members.php?id=' . $group);
+}
+
 function group_removeuser_validate(Pieform $form, $values) {
     global $user, $group, $SESSION;
     if (!group_user_can_leave($values['group'], $values['member'])) {
@@ -618,6 +696,58 @@ function group_removeuser_submit(Pieform $form, $values) {
     redirect('/group/members.php?id=' . $group);
 }
 
+/**
+ * Form for submitting views to a group
+ */
+function group_view_submission_form($groupid, $viewdata) {
+    $options = array();
+    foreach ($viewdata as $view) {
+        if (empty($view->submittedgroup) && empty($view->submittedhost)) {
+            $options[$view->id] = $view->title;
+        }
+    }
+    if (empty($options)) {
+        return;
+    }
+    return pieform(array(
+        'name' => 'group_view_submission_form_' . $groupid,
+        'method' => 'post',
+        'renderer' => 'oneline',
+        'autofocus' => false,
+        'successcallback' => 'group_view_submission_form_submit',
+        'elements' => array(
+            'text1' => array(
+                'type' => 'html', 'value' => get_string('submit', 'group') . ' ',
+            ),
+            'options' => array(
+                'type' => 'select',
+                'collapseifoneoption' => false,
+                'options' => $options,
+            ),
+            'text2' => array(
+                'type' => 'html',
+                'value' => get_string('forassessment', 'view'),
+            ),
+            'submit' => array(
+                'type' => 'submit',
+                'value' => get_string('submit')
+            ),
+            'group' => array(
+                'type' => 'hidden',
+                'value' => $groupid
+            ),
+            'returnto' => array(
+                'type' => 'hidden',
+                'value' => get_config('wwwroot') . 'group/view.php?id=' . $groupid,
+            ),
+        ),
+    ));
+}
+
+function group_view_submission_form_submit(Pieform $form, $values) {
+    redirect('/view/submit.php?id=' . $values['options'] . '&group=' . $values['group'] . '&returnto=group');
+}
+
 // Miscellaneous group related functions
 
 /**
@@ -627,10 +757,10 @@ function group_removeuser_submit(Pieform $form, $values) {
  * @return array
  */
 function group_get_admin_ids($groupid) {
-    return (array)get_column_sql("SELECT member
+    return (array)get_column_sql("SELECT \"member\"
         FROM {group_member}
         WHERE \"group\" = ?
-        AND role = 'admin'", $groupid);
+        AND \"role\" = 'admin'", $groupid);
 }
 
 /**
@@ -640,7 +770,7 @@ function group_get_admin_ids($groupid) {
  * @return array
  */
 function group_get_role_info($groupid) {
-    $roles = get_records_sql_assoc('SELECT role, edit_views, see_submitted_views, gr.grouptype FROM {grouptype_roles} gr
+    $roles = get_records_sql_assoc('SELECT "role", edit_views, see_submitted_views, gr.grouptype FROM {grouptype_roles} gr
         INNER JOIN {group} g ON g.grouptype = gr.grouptype
         WHERE g.id = ?', array($groupid));
     foreach ($roles as $role) {
@@ -673,10 +803,10 @@ function group_prepare_usergroups_for_display($groups, $returnto='mygroups') {
     $groupadmins = array();
     $groupids = array_map(create_function('$a', 'return $a->id;'), $groups);
     if ($groupids) {
-        $groupadmins = get_records_sql_array('SELECT "group", member
+        $groupadmins = get_records_sql_array('SELECT "group", "member"
             FROM {group_member}
             WHERE "group" IN (' . implode(',', db_array_to_ph($groupids)) . ")
-            AND role = 'admin'", $groupids);
+            AND \"role\" = 'admin'", $groupids);
         if (!$groupadmins) {
             $groupadmins = array();
         }
@@ -690,7 +820,6 @@ function group_prepare_usergroups_for_display($groups, $returnto='mygroups') {
                 $group->admins[] = $admin->member;
             }
         }
-        $group->description = str_shorten_html($group->description, 100, true);
         if ($group->membershiptype == 'member') {
             $group->canleave = group_user_can_leave($group->id);
         }
@@ -700,19 +829,75 @@ function group_prepare_usergroups_for_display($groups, $returnto='mygroups') {
         else if ($group->membershiptype == 'invite') {
             $group->invite = group_get_accept_form('invite' . $i++, $group->id, $returnto);
         }
+        $group->settingsdescription = group_display_settings($group);
     }
 }
 
+/*
+ * Used by admin/groups/groups.php and admin/groups/groups.json.php for listing groups.
+ */
+function build_grouplist_html($query, $limit, $offset, &$count=null) {
 
-function group_get_membersearch_data($group, $query, $offset, $limit, $membershiptype) {
+    $groups = search_group($query, $limit, $offset, 'all');
+    $count = $groups['count'];
+
+    if ($ids = array_map(create_function('$a', 'return intval($a->id);'), $groups['data'])) {
+        $sumsql = "(m.role = 'admin')";
+        if (is_postgres()) {
+            $sumsql .= '::int';
+        }
+
+        // Member & admin counts
+        $ids = join(',', $ids);
+        $counts = get_records_sql_assoc("
+            SELECT m.group, COUNT(m.member) AS members, SUM($sumsql) AS admins
+            FROM {group_member} m
+            WHERE m.group IN ($ids)
+            GROUP BY m.group",
+            array()
+        );
+    }
+
+    foreach ($groups['data'] as &$group) {
+        $group->visibility = $group->public ? get_string('Public', 'group') : get_string('Members', 'group');
+        $group->admins = empty($counts[$group->id]->admins) ? 0 : $counts[$group->id]->admins;
+        $group->members = empty($counts[$group->id]->members) ? 0 : $counts[$group->id]->members;
+        if (get_config('allowgroupcategories')) {
+            $group->categorytitle = ($group->category) ? get_field('group_category', 'title', 'id', $group->category) : '';
+        }
+    }
+
+    $smarty = smarty_core();
+    $smarty->assign('groups', $groups['data']);
+    $data = array();
+    $data['tablerows'] = $smarty->fetch('admin/groups/groupsresults.tpl');
+
+    $pagination = build_pagination(array(
+                'id' => 'admgroupslist_pagination',
+                'datatable' => 'admgroupslist',
+                'url' => get_config('wwwroot') . 'admin/groups/groups.php' . (!empty($query) ? '?query=' . urlencode($query) : ''),
+                'jsonscript' => 'admin/groups/groups.json.php',
+                'count' => $count,
+                'limit' => $limit,
+                'offset' => $offset,
+                'resultcounttextsingular' => get_string('group', 'group'),
+                'resultcounttextplural' => get_string('groups', 'group'),
+            ));
+
+    $data['pagination'] = $pagination['html'];
+    $data['pagination_js'] = $pagination['javascript'];
+
+    return $data;
+}
+
+function group_get_membersearch_data($results, $group, $query, $membershiptype) {
     global $USER;
-    $results = get_group_user_search_results($group, $query, $offset, $limit, $membershiptype);
 
     $params = array();
     if (!empty($query)) {
         $params[] = 'query=' . $query;
     }
-    $params[] = 'limit=' . $limit;
+    $params[] = 'limit=' . $results['limit'];
     if (!empty($membershiptype)) {
         $params[] = 'membershiptype=' . $membershiptype;
     }
@@ -740,6 +925,7 @@ function group_get_membersearch_data($group, $query, $offset, $limit, $membershi
         if ($membershiptype == 'request') {
             foreach ($results['data'] as &$r) {
                 $r['addform'] = group_get_adduser_form($r['id'], $group);
+                $r['denyform'] = group_get_denyuser_form($r['id'], $group);
                 // TODO: this will suck when there's quite a few on the page, 
                 // would be better to grab all the reasons in one go
                 $r['reason']  = get_field('group_member_request', 'reason', 'group', $group, 'member', $r['id']);
@@ -762,10 +948,10 @@ function group_get_membersearch_data($group, $query, $offset, $limit, $membershi
         'class' => 'center',
         'url' => $searchurl,
         'count' => $results['count'],
-        'limit' => $limit,
-        'offset' => $offset,
+        'limit' => $results['limit'],
+        'offset' => $results['offset'],
         'datatable' => 'membersearchresults',
-        'jsonscript' => 'group/membersearchresults.php',
+        'jsonscript' => 'group/membersearchresults.json.php',
         'firsttext' => '',
         'previoustext' => '',
         'nexttext' => '',
@@ -775,7 +961,7 @@ function group_get_membersearch_data($group, $query, $offset, $limit, $membershi
         'resultcounttextplural' => get_string('members', 'group'),
     ));
 
-    return array($html, $pagination, $results['count'], $offset, $membershiptype);
+    return array($html, $pagination, $results['count'], $results['offset'], $membershiptype);
 }
 
 
@@ -870,6 +1056,15 @@ function group_get_menu_tabs() {
         'weight' => 50,
     );
 
+    if (group_user_can_edit_views($group->id)) {
+        $menu['share'] = array(
+            'path' => 'groups/share',
+            'url' => 'group/shareviews.php?group='.$group->id,
+            'title' => get_string('share', 'view'),
+            'weight' => 60,
+        );
+    }
+
     if (group_user_access($group->id)) {
         safe_require('grouptype', $group->grouptype);
         $artefactplugins = call_static_method('GroupType' . $group->grouptype, 'get_group_artefact_plugins');
@@ -991,7 +1186,7 @@ function group_sideblock() {
 }
 
 
-function group_get_associated_groups($userid, $filter='all', $limit=20, $offset=0) {
+function group_get_associated_groups($userid, $filter='all', $limit=20, $offset=0, $category='') {
 
     // Strangely, casting is only needed for invite, request and admin and only in 
     // postgres
@@ -1007,6 +1202,7 @@ function group_get_associated_groups($userid, $filter='all', $limit=20, $offset=
         $adminsql   = "CAST('admin' AS TEXT)";
         $empty      = "CAST('' AS TEXT)";
     }
+    // TODO: make it work on other databases?
 
     // Different filters join on the different kinds of association
     if ($filter == 'admin') {
@@ -1073,69 +1269,71 @@ function group_get_associated_groups($userid, $filter='all', $limit=20, $offset=
     
     $values[] = 0;
     
-    $count = count_records_sql('SELECT COUNT(*) FROM {group} g ' . $sql . ' WHERE g.deleted = ?', $values);
+    $catsql = '';
+    if (!empty($category)) {
+        if ($category == -1) { //find unassigned groups
+            $catsql = ' AND g.category IS NULL';
+        } else {
+            $catsql = ' AND g.category = ?';
+            $values[] = $category;
+        }
+    }
+
+    $count = count_records_sql('SELECT COUNT(*) FROM {group} g ' . $sql . ' WHERE g.deleted = ?'.$catsql, $values);
     
     // almost the same as query used in find - common parts should probably be pulled out
     // gets the groups filtered by above
     // and the first three members by id
     
-    $sql = 'SELECT g1.id, g1.name, g1.description, g1.jointype, g1.grouptype, g1.membershiptype, g1.reason, g1.role, g1.membercount, COUNT(gmr.member) AS requests
+    $sql = 'SELECT g1.id, g1.name, g1.description, g1.public, g1.jointype, g1.grouptype, g1.membershiptype, g1.reason, g1.role, g1.membercount, COUNT(gmr.member) AS requests
         FROM (
-        SELECT g.id, g.name, g.description, g.jointype, g.grouptype, t.membershiptype, t.reason, t.role, COUNT(gm.member) AS membercount
+        SELECT g.id, g.name, g.description, g.public, g.jointype, g.grouptype, t.membershiptype, t.reason, t.role, COUNT(gm.member) AS membercount
             FROM {group} g
             LEFT JOIN {group_member} gm ON (gm.group = g.id)' .
             $sql . '
-            WHERE g.deleted = ?
-            GROUP BY g.id, g.name, g.description, g.jointype, g.grouptype, t.membershiptype, t.reason, t.role
-            ORDER BY g.name
+            WHERE g.deleted = ?' .
+            $catsql . '
+            GROUP BY g.id, g.name, g.description, g.public, g.jointype, g.grouptype, t.membershiptype, t.reason, t.role
         ) g1
         LEFT JOIN {group_member_request} gmr ON (gmr.group = g1.id)
-        GROUP BY g1.id, g1.name, g1.description, g1.jointype, g1.grouptype, g1.membershiptype, g1.reason, g1.role, g1.membercount';
-    
-    $groups = get_records_sql_assoc($sql, $values, $offset, $limit);
-    
-    if ($groups) {
-        // Get a few random members from each group. We've tried this with one 
-        // query before but it's painfully slow, databases don't do random rows 
-        // efficiently.
-        foreach (array_keys($groups) as $groupid) {
-            $members = get_records_sql_array("
-                SELECT u.*
-                FROM {group_member} gm
-                INNER JOIN {usr} u ON (gm.member = u.id AND u.deleted = 0)
-                WHERE gm.group = ?
-                ORDER BY " . db_random() . "
-                LIMIT 3", array($groupid));
-            foreach ($members as $m) {
-                $groups[$groupid]->members[] = (object) array('id' => $m->id, 'name' => display_name($m));
-            }
-        }
-        $groups = array_values($groups);
-    }
-    else {
-        $groups = array();
-    }
+        GROUP BY g1.id, g1.name, g1.description, g1.public, g1.jointype, g1.grouptype, g1.membershiptype, g1.reason, g1.role, g1.membercount
+        ORDER BY g1.name';
 
-    return array('groups' => $groups, 'count' => $count);
+    $groups = get_records_sql_array($sql, $values, $offset, $limit);
+    
+    return array('groups' => $groups ? $groups : array(), 'count' => $count);
 
 }
 
 
 function group_get_user_groups($userid=null, $roles=null) {
+    static $usergroups = array();
+
     if (is_null($userid)) {
         global $USER;
         $userid = $USER->get('id');
     }
-    if ($groups = get_records_sql_array(
-        "SELECT g.id, g.name, gm.role, g.jointype, g.grouptype
+
+    if (empty($roles) && isset($usergroups[$userid])) {
+        return $usergroups[$userid];
+    }
+
+    if (!$groups = get_records_sql_array(
+        "SELECT g.id, g.name, gm.role, g.jointype, g.grouptype, gtr.see_submitted_views, g.category
         FROM {group} g
         JOIN {group_member} gm ON (gm.group = g.id)
+        JOIN {grouptype_roles} gtr ON (g.grouptype = gtr.grouptype AND gm.role = gtr.role)
         WHERE gm.member = ?
         AND g.deleted = 0 " . (is_array($roles) ? (' AND gm.role IN (' . join(',', array_map('db_quote', $roles)) . ')') : '') . "
-        ORDER BY gm.role = 'admin' DESC, gm.role, g.id", array($userid))) {
-        return $groups;
+        ORDER BY g.name, gm.role = 'admin' DESC, gm.role, g.id", array($userid))) {
+        $groups = array();
     }
-    return array();
+
+    if (empty($roles)) {
+        $usergroups[$userid] = $groups;
+    }
+
+    return $groups;
 }
 
 
@@ -1160,4 +1358,122 @@ function group_can_create_groups() {
     }
     return $creators == 'staff' && ($USER->get('staff') || $USER->is_institutional_staff());
 }
-?>
+
+function group_get_user_course_groups($userid=null) {
+    if (is_null($userid)) {
+        global $USER;
+        $userid = $USER->get('id');
+    }
+    if ($groups = get_records_sql_array(
+        "SELECT g.id, g.name
+        FROM {group_member} u
+        INNER JOIN {group} g ON (u.group = g.id AND g.deleted = 0)
+        INNER JOIN {grouptype} t ON t.name = g.grouptype
+        WHERE u.member = ?
+        AND t.submittableto = 1
+        ORDER BY g.name
+        ", array($userid))) {
+        return $groups;
+    }
+    return array();
+}
+
+function group_allows_submission($grouptype) {
+    static $grouptypes = null;
+    if (is_null($grouptypes)) {
+        $grouptypes = get_records_assoc('grouptype');
+    }
+    return (bool) $grouptypes[$grouptype]->submittableto;
+}
+
+function group_display_settings($group) {
+    $string = get_string('membershiptype.'.$group->jointype, 'group');
+    if (group_allows_submission($group->grouptype)) {
+        $string .= ', ' . get_string('allowssubmissions', 'group');
+    }
+    if ($group->public) {
+        $string .= ', ' . get_string('publiclyvisible', 'group');
+    }
+    return $string;
+}
+
+/**
+ * Return the view object for this group's homepage view
+ *
+ * @param int $groupid the id of the group to fetch the view for
+ *
+ * @throws ViewNotFoundException
+ */
+function group_get_homepage_view($groupid) {
+    $v = get_record('view', 'group', $groupid, 'type', 'grouphomepage');
+    return new View($v->id, (array)$v);
+}
+
+/**
+ * install the group homepage view
+ * This creates a template at system level
+ * which is subsequently copied to group hompages
+ *
+ * @return int the id of the new template
+ */
+function install_system_grouphomepage_view() {
+    $dbtime = db_format_timestamp(time());
+    // create a system template for group homepage views
+    require_once(get_config('libroot') . 'view.php');
+    $viewdata = (object) array(
+        'type'        => 'grouphomepage',
+        'owner'       => 0,
+        'numcolumns'  => 1,
+        'template'    => 1,
+        'title'       => get_string('grouphomepage', 'view'),
+        'ctime'       => $dbtime,
+        'atime'       => $dbtime,
+        'mtime'       => $dbtime,
+    );
+    $id = insert_record('view', $viewdata, 'id', true);
+    $accessdata = (object) array('view' => $id, 'accesstype' => 'loggedin');
+    insert_record('view_access', $accessdata);
+    $blocktypes = array(
+        array(
+            'blocktype' => 'groupinfo',
+            'title' => '',
+            'column' => 1,
+            'config' => null,
+        ),
+        array(
+            'blocktype' => 'recentforumposts',
+            'title' => '',
+            'column' => 1,
+            'config' => null,
+        ),
+        array(
+            'blocktype' => 'groupviews',
+            'title' => '',
+            'column' => 1,
+            'config' => null,
+        ),
+        array(
+            'blocktype' => 'groupmembers',
+            'title' => '',
+            'column' => 1,
+            'config' => null,
+        ),
+    );
+    $installed = get_column_sql('SELECT name FROM {blocktype_installed}');
+    $weights = array(1 => 0);
+    foreach ($blocktypes as $blocktype) {
+        if (in_array($blocktype['blocktype'], $installed)) {
+            $weights[$blocktype['column']]++;
+            insert_record('block_instance', (object) array(
+                'blocktype'  => $blocktype['blocktype'],
+                'title'      => $blocktype['title'],
+                'view'       => $id,
+                'column'     => $blocktype['column'],
+                'order'      => $weights[$blocktype['column']],
+                'configdata' => serialize($blocktype['config']),
+            ));
+        }
+    }
+
+    return $id;
+}
