@@ -28,7 +28,6 @@
 // TODO : lib
 
 defined('INTERNAL') || die();
-require_once(get_config('libroot') .'hostset.php');
 
 class Institution {
 
@@ -37,7 +36,6 @@ class Institution {
     const   PERSISTENT     = 2;
 
     protected $initialized = self::UNINITIALIZED;
-    protected $hostset;
     protected $members = array(
         'name' => '',
         'displayname' => '',
@@ -58,11 +56,6 @@ class Institution {
     }
 
     function __get($name) {
-        if ($name == 'hosts') {
-            if (count($this->hosts) == 0 && $this->ready) {
-                $this->getHostData();
-            }
-        }
         if (array_key_exists($name, $this->members)) {
             return $this->members[$name];
         }
@@ -119,33 +112,6 @@ class Institution {
         return $this;
     }
 
-    function findByWwwroot($wwwroot) {
-
-        if (!is_string($wwwroot) || strlen($wwwroot) < 1 || strlen($wwwroot) > 255) {
-            throw new SystemException('Invalid wwwroot: ' . $wwwroot);
-        }
-
-        $this->hostset = new HostSet();
-        $this->hostset->findByWwwroot($wwwroot);
-        if (false == $a_host = $this->hostset->current()) {
-            return false;
-        }
-        $institution = $a_host->institution;
-
-        $result = get_record('institution', 'name', $institution);
-        if (false == $result) {
-            throw new SystemException('Invalid Institution name');
-        }
-
-        $this->initialized = self::PERSISTENT;
-        $this->populate($result);
-    }
-
-    function getHostData() {
-        $this->hostset = new HostSet();
-        $this->hostset->findByInstitution($this->name);
-    }
-
     function initialise($name, $displayname) {
         if (empty($name) || !is_string($name)) {
             return false;
@@ -174,7 +140,7 @@ class Institution {
 
     function commit() {
         if (!$this->verifyReady()) {
-            throw new Exception();
+            throw new SystemException('Commit failed');
         }
 
         $record = new stdClass();
@@ -193,18 +159,6 @@ class Institution {
         return false;
     }
 
-    function findByHostset(HostSet $hostset) {
-
-        // get the first host record:
-        $host = reset($hostset);
-
-        $result = get_record('institution', 'name', $host->institution);
-        if (false == $result) {
-            throw new SystemException('No institution for hostset '.addslashes($host->institution));
-        }
-        $this->populate($result);
-    }
-
     protected function populate($result) {
         $this->name                         = $result->name;
         $this->displayname                  = $result->displayname;
@@ -340,7 +294,7 @@ class Institution {
     public function removeMembers($userids) {
         // Remove self last.
         global $USER;
-        $users = get_records_select_array('usr', 'id IN (' . join(',', $userids) . ')');
+        $users = get_records_select_array('usr', 'id IN (' . join(',', array_map('intval', $userids)) . ')');
         $removeself = false;
         foreach ($users as $user) {
             if ($user->id == $USER->id) {
@@ -360,9 +314,14 @@ class Institution {
         }
         db_begin();
         // If the user is being authed by the institution they are
-        // being removed from, change them to internal auth
-        $authinstances = get_records_select_assoc('auth_instance', "
-            institution IN ('mahara','" . $this->name . "')");
+        // being removed from, change them to internal auth, or if
+        // we can't find that, some other no institution auth.
+        $authinstances = get_records_select_assoc(
+            'auth_instance',
+            "institution IN ('mahara', ?)",
+            array($this->name),
+            "institution = 'mahara' DESC, authname = 'internal' DESC"
+        );
         $oldauth = $user->authinstance;
         if (isset($authinstances[$oldauth]) && $authinstances[$oldauth]->institution == $this->name) {
             foreach ($authinstances as $ai) {
@@ -370,6 +329,10 @@ class Institution {
                     $user->authinstance = $ai->id;
                     break;
                 }
+                else if ($ai->institution == 'mahara') {
+                    $user->authinstance = $ai->id;
+                    break;
+                }
             }
             delete_records('auth_remote_user', 'authinstance', $oldauth, 'localusr', $user->id);
             // If the old authinstance was external, the user may need
@@ -437,6 +400,95 @@ class Institution {
     public function isFull() {
         return ($this->maxuseraccounts != '') && ($this->countMembers() >= $this->maxuseraccounts);
     }
+
+    /**
+     * Returns the list of institutions, implements institution searching
+     *
+     * @param array   Limit the output to only institutions in this array (used for institution admins).
+     * @param bool    Whether default institution should be listed in results.
+     * @param string  Searching query string.
+     * @param int     Limit of results (used for pagination).
+     * @param int     Offset of results (used for pagination).
+     * @param int     Returns the total number of results.
+     * @return array  A data structure containing results looking like ...
+     *   $institutions = array(
+     *                       name => array(
+     *                           displayname     => string
+     *                           maxuseraccounts => integer
+     *                           members         => integer
+     *                           staff           => integer
+     *                           admins          => integer
+     *                           name            => string
+     *                       ),
+     *                       name => array(...),
+     *                   );
+     */
+    public static function count_members($filter, $showdefault, $query='', $limit=null, $offset=null, &$count=null) {
+        if ($filter) {
+            $where = '
+            AND ii.name IN (' . join(',', array_map('db_quote', $filter)) . ')';
+        }
+        else {
+            $where = '';
+        }
+
+        $querydata = split(' ', preg_replace('/\s\s+/', ' ', strtolower(trim($query))));
+        $namesql = '(
+                ii.name ' . db_ilike() . ' \'%\' || ? || \'%\'
+            )
+            OR (
+                ii.displayname ' . db_ilike() . ' \'%\' || ? || \'%\'
+            )';
+        $namesql = join(' OR ', array_fill(0, count($querydata), $namesql));
+        $queryvalues = array();
+        foreach ($querydata as $w) {
+            $queryvalues = array_pad($queryvalues, count($queryvalues) + 2, $w);
+        }
+
+        $count = count_records_sql('SELECT COUNT(ii.name)
+            FROM {institution} ii
+            WHERE' . $namesql, $queryvalues
+        );
+
+        $institutions = get_records_sql_assoc('
+            SELECT
+                ii.name,
+                ii.displayname,
+                ii.maxuseraccounts,
+                ii.suspended,
+                COALESCE(a.members, 0) AS members,
+                COALESCE(a.staff, 0) AS staff,
+                COALESCE(a.admins, 0) AS admins
+            FROM
+                {institution} ii
+                LEFT JOIN
+                    (SELECT
+                        i.name, i.displayname, i.maxuseraccounts,
+                        COUNT(ui.usr) AS members, SUM(ui.staff) AS staff, SUM(ui.admin) AS admins
+                    FROM
+                        {institution} i
+                        LEFT OUTER JOIN {usr_institution} ui ON (ui.institution = i.name)
+                        LEFT OUTER JOIN {usr} u ON (u.id = ui.usr)
+                    WHERE
+                        (u.deleted = 0 OR u.id IS NULL)
+                    GROUP BY
+                        i.name, i.displayname, i.maxuseraccounts
+                    ) a ON (a.name = ii.name)
+                    WHERE (' . $namesql . ')' . $where . '
+                    ORDER BY
+                        ii.name = \'mahara\', ii.displayname', $queryvalues, $offset, $limit);
+
+        if ($showdefault && $institutions && array_key_exists('mahara', $institutions)) {
+            $defaultinstmembers = count_records_sql('
+                SELECT COUNT(u.id) FROM {usr} u LEFT OUTER JOIN {usr_institution} i ON u.id = i.usr
+                WHERE u.deleted = 0 AND i.usr IS NULL AND u.id != 0
+            ');
+            $institutions['mahara']->members = $defaultinstmembers;
+            $institutions['mahara']->staff   = '';
+            $institutions['mahara']->admins  = '';
+        }
+        return $institutions;
+    }
 }
 
 function get_institution_selector($includedefault = true) {
@@ -444,14 +496,17 @@ function get_institution_selector($includedefault = true) {
 
     if ($USER->get('admin')) {
         if ($includedefault) {
-            $institutions = get_records_array('institution');
+            $institutions = get_records_array('institution', '', '', 'displayname');
         }
         else {
-            $institutions = get_records_select_array('institution', "name != 'mahara'");
+            $institutions = get_records_select_array('institution', "name != 'mahara'", null, 'displayname');
         }
     } else if ($USER->is_institutional_admin()) {
-        $institutions = get_records_select_array('institution', 'name IN (' 
-            . join(',', array_map('db_quote',$USER->get('admininstitutions'))) . ')');
+        $institutions = get_records_select_array(
+            'institution',
+            'name IN (' . join(',', array_map('db_quote',$USER->get('admininstitutions'))) . ')',
+            null, 'displayname'
+        );
     } else {
         return null;
     }
@@ -523,4 +578,31 @@ EOF;
         'institutionselectorjs' => $js
     );
 }
-?>
+
+function build_institutions_html($filter, $showdefault, $query, $limit, $offset, &$count=null) {
+    global $USER;
+
+    $institutions = Institution::count_members($filter, $showdefault, $query, $limit, $offset, $count);
+
+    $smarty = smarty_core();
+    $smarty->assign('institutions', $institutions);
+    $smarty->assign('siteadmin', $USER->get('admin'));
+    $data['tablerows'] = $smarty->fetch('admin/users/institutionsresults.tpl');
+
+    $pagination = build_pagination(array(
+                'id' => 'adminstitutionslist_pagination',
+                'datatable' => 'adminstitutionslist',
+                'url' => get_config('wwwroot') . 'admin/users/institutions.php' . (!empty($query) ? '?query=' . urlencode($query) : ''),
+                'jsonscript' => 'admin/users/institutions.json.php',
+                'count' => $count,
+                'limit' => $limit,
+                'offset' => $offset,
+                'resultcounttextsingular' => get_string('institution', 'admin'),
+                'resultcounttextplural' => get_string('institutions', 'admin'),
+            ));
+
+    $data['pagination'] = $pagination['html'];
+    $data['pagination_js'] = $pagination['javascript'];
+
+    return $data;
+}
