@@ -71,15 +71,6 @@ function pieform($data) {/*{{{*/
     return Pieform::process($data);
 }/*}}}*/
 
-// json_encode replacement, if the user doesn't have this function available
-if (!function_exists('json_encode')) {/*{{{*/
-    function json_encode($data) {
-        require_once 'JSON/JSON.php';
-        $json = new Services_JSON();
-        return $json->encode($data);
-    }
-}/*}}}*/
-
 /**
  * Pieforms throws PieformExceptions when things go wrong
  */
@@ -201,7 +192,8 @@ class Pieform {/*{{{*/
         }
 
         // Assign defaults for the form
-        $this->data = array_merge(self::get_pieform_defaults(), $formconfig, $data);
+        $this->defaults = self::get_pieform_defaults();
+        $this->data = array_merge($this->defaults, $formconfig, $data);
 
         // Set the method - only get/post allowed
         $this->data['method'] = strtolower($this->data['method']);
@@ -239,6 +231,90 @@ class Pieform {/*{{{*/
             throw new PieformException('Forms must have a list of elements');
         }
 
+        if (isset($this->data['spam'])) {
+            // Enable form tricks to make it harder for bots to fill in the form.
+            // This was moved from lib/antispam.php, see:
+            // http://wiki.mahara.org/Developer_Area/Specifications_in_Development/Anti-spam#section_7
+            //
+            // Use the spam_error() method in your _validate function to check whether a submitted form
+            // has failed any of these checks.
+            //
+            // Available options:
+            //  - hash:    An array of element names to be hashed.  Currently ids of input elements
+            //             are also hashed, so you need to be careful if you include 'elementname' in
+            //             the hash array, and make sure you rewrite any css or js so it doesn't rely on
+            //             an id like 'formname_elementname'.
+            //  - secret:  String used to hash the fields.
+            //  - mintime: Minimum number of seconds that must pass between page load & form submission.
+            //  - maxtime: Maximum number of seconds that must pass between page load & form submission.
+            //  - reorder: Array of element names to be reordered at random.
+            if (empty($this->data['spam']['secret']) || !isset($this->data['elements']['submit'])) {
+                // @todo don't rely on submit element
+                throw new PieformException('Forms with spam config must have a secret and submit element');
+            }
+            $this->time = isset($_POST['__timestamp']) ? $_POST['__timestamp'] : time();
+            $spamelements1 = array(
+                '__invisiblefield' => array(
+                    'type'         => 'text',
+                    'title'        => get_string('spamtrap'),
+                    'defaultvalue' => '',
+                    'class'        => 'dontshow',
+                ),
+            );
+            $spamelements2 = array(
+                '__timestamp' => array(
+                    'type' => 'hidden',
+                    'value' => $this->time,
+                ),
+                '__invisiblesubmit' => array(
+                    'type'  => 'submit',
+                    'value' => get_string('spamtrap'),
+                    'class' => 'dontshow',
+                ),
+            );
+            $insert = rand(0, count($this->data['elements']));
+            $this->data['elements'] = array_merge(
+                array_slice($this->data['elements'], 0, $insert, true),
+                $spamelements1,
+                array_slice($this->data['elements'], $insert, count($this->data['elements']) - $insert, true),
+                $spamelements2
+            );
+
+            // Min & max number of seconds between page load & submission
+            if (!isset($this->data['spam']['mintime'])) {
+                $this->data['spam']['mintime'] = 0.01;
+            }
+            if (!isset($this->data['spam']['maxtime'])) {
+                $this->data['spam']['maxtime'] = 86400;
+            }
+
+            if (empty($this->data['spam']['hash'])) {
+                $this->data['spam']['hash'] = array();
+            }
+            $this->data['spam']['hash'][] = '__invisiblefield';
+            $this->data['spam']['hash'][] = '__invisiblesubmit';
+            $this->hash_fieldnames();
+
+            if (isset($this->data['spam']['reorder'])) {
+                // Reorder form fields randomly
+                $order = $this->data['spam']['reorder'];
+                shuffle($order);
+                $order = array_combine($this->data['spam']['reorder'], $order);
+                $temp = array();
+                foreach (array_keys($this->data['elements']) as $k) {
+                    if (isset($order[$k])) {
+                        $temp[$order[$k]] = $this->data['elements'][$order[$k]];
+                    }
+                    else {
+                        $temp[$k] = $this->data['elements'][$k];
+                    }
+                }
+                $this->data['elements'] = $temp;
+            }
+
+            $this->spamerror = false;
+        }
+
         // Get references to all the elements in the form, excluding fieldsets
         foreach ($this->data['elements'] as $name => &$element) {
             // The name can be in the element itself. This is compatibility for 
@@ -267,7 +343,8 @@ class Pieform {/*{{{*/
                 $this->elementrefs[$name] = &$element;
             }
 
-            $element['name'] = $name;
+            $element['name'] = isset($this->hashedfields[$name]) ? $this->hashedfields[$name] : $name;
+
         }
         unset($element);
 
@@ -360,6 +437,11 @@ class Pieform {/*{{{*/
                     $element['autofocus'] = true;
                 }
 
+                if (!empty($element['autofocus']) && $element['type'] == 'text' && !empty($this->data['autoselect'])
+                    && $name == $this->data['autoselect']) {
+                    $element['autoselect'] = true;
+                }
+
                 // All elements inherit the form tabindex
                 $element['tabindex'] = $this->data['tabindex'];
             }
@@ -412,9 +494,9 @@ class Pieform {/*{{{*/
             // Submit the form if things went OK
             if ($this->data['submit'] && !$this->has_errors()) {
                 $submitted = false;
-                foreach ($this->elementrefs as $element) {
+                foreach ($this->elementrefs as $name => $element) {
                     if (!empty($element['submitelement']) && isset($global[$element['name']])) {
-                        $function = "{$this->data['successcallback']}_{$element['name']}";
+                        $function = "{$this->data['successcallback']}_{$name}";
                         if (function_exists($function)) {
                             $function($this, $values);
                             $submitted = true;
@@ -530,17 +612,20 @@ class Pieform {/*{{{*/
             $result .= ' error';
         }
         if (isset($this->data['class'])) {
-            $result .= ' ' . $this->data['class'];
+            $result .= ' ' . self::hsc($this->data['class']);
         }
         $result .= '"';
         foreach (array('name', 'method', 'action') as $attribute) {
-            $result .= ' ' . $attribute . '="' . $this->data[$attribute] . '"';
+            $result .= ' ' . $attribute . '="' . self::hsc($this->data[$attribute]) . '"';
         }
         $result .= ' id="' . $this->name . '"';
         if ($this->fileupload) {
             $result .= ' enctype="multipart/form-data"';
         }
         $result .= '>';
+        if (!empty($this->error)) {
+            $result .= '<div class="error">' . $this->error . '</div>';
+        }
         return $result;
     }/*}}}*/
 
@@ -814,6 +899,9 @@ class Pieform {/*{{{*/
         else if (is_string($replacehtml)) {
             $data['replaceHTML'] = $replacehtml;
         }
+        if (isset($this->hashedfields)) {
+            $data['fieldnames'] = $this->hashedfields;
+        }
 
         $result = json_encode($data);
 
@@ -857,7 +945,11 @@ EOF;
      * @throws PieformException  If the element could not be found
      */
     public function set_error($name, $message) {/*{{{*/
-        foreach ($this->data['elements'] as &$element) {
+        if (is_null($name) && !empty($message)) {
+            $this->error = $message;
+            return;
+        }
+        foreach ($this->data['elements'] as $key => &$element) {
             if ($element['type'] == 'fieldset') {
                 foreach ($element['elements'] as &$subelement) {
                     if ($subelement['name'] == $name) {
@@ -867,7 +959,7 @@ EOF;
                 }
             }
             else {
-                if ($element['name'] == $name) {
+                if ($key == $name) {
                     $element['error'] = $message;
                     return;
                 }
@@ -887,7 +979,7 @@ EOF;
                 return true;
             }
         }
-        return false;
+        return isset($this->error);
     }/*}}}*/
 
     /**
@@ -925,7 +1017,7 @@ EOF;
         if (isset($element['name'])) {
             return self::hsc($element['name']);
         }
-        return substr(md5(mt_rand()), 0, 4);
+        return 'a' . substr(md5(mt_rand()), 0, 4);
     }/*}}}*/
 
     /**
@@ -943,7 +1035,7 @@ EOF;
     public function make_class($element) {/*{{{*/
         $classes = array();
         if (isset($element['class'])) {
-            $classes[] = $element['class'];
+            $classes[] = self::hsc($element['class']);
         }
         if (!empty($element['rules']['required'])) {
             $classes[] = 'required';
@@ -954,6 +1046,9 @@ EOF;
         if ($this->data['elementclasses']) {
             $classes[] = $element['type'];
         }
+        if (!empty($element['autoselect'])) {
+            $classes[] = 'autoselect';
+        }
         // Please make sure that 'autofocus' is the last class added in this
         // method. Otherwise, improve the logic for removing 'autofocus' from
         // the element class string in pieform_render_element
@@ -1028,6 +1123,10 @@ EOF;
             throw new PieformException("The type \"$type\" is not allowed for an include plugin");
         }
 
+        if (!isset($name) || !preg_match('/^[a-z_][a-z0-9_]*$/', $name)) {
+            throw new PieformException("The name \"$name\" is not valid (validity test: could you give a PHP function the name?)");
+        }
+
         // Check the configured include paths if they are specified
         foreach ($this->data['configdirs'] as $directory) {
             $file = "$directory/{$type}s/$name.php";
@@ -1050,7 +1149,7 @@ EOF;
     /**
      * Return an internationalised string based on the passed input key
      *
-     * Returns english by default.
+     * Returns English by default.
      *
      * @param string $plugin     The type of plugin (element, renderer, rule)
      * @param string $pluginname The name of the plugin to get the language
@@ -1065,6 +1164,14 @@ EOF;
             throw new PieformException("Invalid plugin name '$plugin'");
         }
 
+        if (!isset($pluginname) || !preg_match('/^[a-z_][a-z0-9_]*$/', $pluginname)) {
+            throw new PieformException("The pluginname \"$pluginname\" is not valid (validity test: could you give a PHP function the name?)");
+        }
+
+        if (!isset($key) || !preg_match('/^[a-z_][a-z0-9_]*$/', $key)) {
+            throw new PieformException("The key \"$key\" is not valid (validity test: could you give a PHP function the name?)");
+        }
+
         // Check the element itself for the language string
         if ($plugin == 'rule' && isset($element['rulei18n'][$key])) {
             return $element['rulei18n'][$key];
@@ -1075,19 +1182,8 @@ EOF;
             return $this->data['rulei18n'][$key];
         }
 
-        // Fall back to the default string
-        $this->include_plugin($plugin, $pluginname);
-        $function = 'pieform_' . $plugin . '_' . $pluginname . '_i18n';
-        if (function_exists($function)) {
-            $strings = $function();
-            if (isset($strings[$this->data['language']][$key])) {
-                return $strings[$this->data['language']][$key];
-            }
-            return '[[' . $key . '/' . $this->data['language'] . ']]';
-        }
+        return get_raw_string($plugin . '.' . $pluginname . '.' . $key, 'pieforms');
 
-        // We don't recognise this string
-        return '[[' . $key . ']]';
     }/*}}}*/
 
     /**
@@ -1143,7 +1239,7 @@ EOF;
     private function get_submitted_values() {/*{{{*/
         $result = array();
         $global = ($this->data['method'] == 'get') ? $_GET : $_POST;
-        foreach ($this->elementrefs as $element) {
+        foreach ($this->elementrefs as $name => $element) {
             if ($element['type'] != 'markup') {
                 if (
                     (empty($element['submitelement']) && empty($element['cancelelement'])) ||
@@ -1152,7 +1248,7 @@ EOF;
                         && isset($global[$element['name']])
                     )
                 ) {
-                    $result[$element['name']] = $this->get_value($element);
+                    $result[$name] = $this->get_value($element);
                 }
             }
         }
@@ -1179,10 +1275,10 @@ EOF;
         }
 
         // Perform rule validation
-        foreach ($this->elementrefs as $element) {
+        foreach ($this->elementrefs as $name => $element) {
             if (isset($element['rules']) && is_array($element['rules'])) {
                 foreach ($element['rules'] as $rule => $data) {
-                    if (!$this->get_error($element['name'])) {
+                    if (!$this->get_error($name)) {
                         // See if this element has a function that describes
                         // how this rule should apply to it
                         $function = 'pieform_element_' . $element['type'] . '_rule_' . $rule;
@@ -1196,14 +1292,30 @@ EOF;
                                 }
                             }
                         }
-                        if ($error = $function($this, $values[$element['name']], $element, $data)) {
-                            $this->set_error($element['name'], $error);
+                        if ($error = $function($this, $values[$name], $element, $data)) {
+                            $this->set_error($name, $error);
                         }
                     }
                 }
             }
         }
 
+        if (isset($this->data['spam'])) {
+            // make sure the user waited long enough but not too long before submitting the form
+            $elapsed = time() - $values['__timestamp'];
+            if ($elapsed < $this->data['spam']['mintime'] || $elapsed > $this->data['spam']['maxtime']) {
+                $this->spamerror = true;
+            }
+            // make sure the real submit button was used. If it wasn't, it won't exist.
+            else if (!isset($values['submit']) || isset($values['__invisiblesubmit'])) {
+                $this->spamerror = true;
+            }
+            // make sure the invisible field is empty
+            else if (!isset($values['__invisiblefield']) || $values['__invisiblefield'] != '') {
+                $this->spamerror = true;
+            }
+        }
+
         // Then user specific validation if a function is available for that
         $function = $this->data['validatecallback'];
         if (is_callable($function)) {
@@ -1424,6 +1536,31 @@ EOF;
         );
     }/*}}}*/
 
+    private function hash_fieldnames() {/*{{{*/
+        // Mess with field names to make it harder for bots to fill in the form
+        $ip = self::get_ip();
+        $secret = $this->data['spam']['secret'];
+        $this->hashedfields = array();
+        foreach ($this->data['spam']['hash'] as $name) {
+            // prefix the hash with an underscore to ensure it is always a valid pieforms element name
+            $this->hashedfields[$name] = '_' . sha1($name . $this->time . $ip . $secret);
+        }
+    }/*}}}*/
+
+    private static function get_ip() {/*{{{*/
+        if (!empty($_SERVER['HTTP_CLIENT_IP'])) {
+            return $_SERVER['HTTP_CLIENT_IP'];
+        }
+        if (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
+            return $_SERVER['HTTP_X_FORWARDED_FOR'];
+        }
+        return $_SERVER['REMOTE_ADDR'];
+    }/*}}}*/
+
+    public function spam_error() {/*{{{*/
+        return $this->spamerror;
+    }/*}}}*/
+
 }/*}}}*/
 
 
@@ -1493,6 +1630,10 @@ function pieform_get_headdata() {/*{{{*/
             }
         }
     }
+    // Fieldsets don't appear in $form->get_elements(), so get their headdata
+    if (!empty($GLOBALS['_PIEFORM_FIELDSETS'])) {
+        $htmlelements = array_merge($htmlelements, pieform_element_fieldset_get_headdata());
+    }
 
     // TODO: jsdirectory should be independent of ANY form
     if ($GLOBALS['_PIEFORM_REGISTRY']) {
@@ -1504,5 +1645,3 @@ function pieform_get_headdata() {/*{{{*/
 
     return array_unique($htmlelements);
 }/*}}}*/
-
-?>
