@@ -31,14 +31,27 @@ require_once(get_config('docroot') . 'notification/lib.php');
 
 class PluginNotificationEmail extends PluginNotification {
 
+    static $userdata = array('htmlmessage', 'emailmessage');
+
     public static function notify_user($user, $data) {
 
         $messagehtml = null;
 
         if (!empty($data->overridemessagecontents)) {
             $subject = $data->subject;
-            $messagebody = $data->message;
-            if (!empty($user->htmlmessage)) {
+            if (!empty($data->emailmessage)) {
+                $messagebody = $data->emailmessage;
+            }
+            else if (!empty($user->emailmessage)) {
+                $messagebody = $user->emailmessage;
+            }
+            else {
+                $messagebody = $data->message;
+            }
+            if (!empty($data->htmlmessage)) {
+                $messagehtml = $data->htmlmessage;
+            }
+            else if (!empty($user->htmlmessage)) {
                 $messagehtml = $user->htmlmessage;
             }
         }
@@ -84,5 +97,3 @@ class PluginNotificationEmail extends PluginNotification {
         email_user($user, $userfrom, $subject, $messagebody, $messagehtml, !empty($data->customheaders) ? $data->customheaders : null);
     }
 }
-
-?>
