@@ -102,17 +102,22 @@ class PluginSearchInternal extends PluginSearch {
                     (gm.member = u.id AND gm.group = ' . (int)$data['group'] . $groupadminsql . ")\n";
             $sql .= $groupjoin;
         }
+        $sql .= "
+                LEFT OUTER JOIN {usr_account_preference} h ON (u.id = h.usr AND h.field = 'hiderealname')";
         $querydata = split(' ', preg_replace('/\s\s+/', ' ', strtolower(trim($query_string))));
+        $hidenameallowed = get_config('userscanhiderealnames') ? 'TRUE' : 'FALSE';
+        $searchusernamesallowed = get_config('searchusernames') ? 'TRUE' : 'FALSE';
         $namesql = "(u.preferredname $ilike '%' || ? || '%')
-                    OR ((u.preferredname IS NULL OR u.preferredname = '')
+                    OR ((u.preferredname IS NULL OR u.preferredname = '' OR NOT $hidenameallowed OR h.value != '1')
                         AND (u.firstname $ilike '%' || ? || '%' OR u.lastname $ilike '%' || ? || '%'))
+                    OR ($searchusernamesallowed AND u.username $ilike '%' || ? || '%')
                     OR (a.artefacttype IN $fieldlist
                         AND ( a.title $ilike '%' || ? || '%'))";
         $namesql = join('
                     OR ', array_fill(0, count($querydata), $namesql));
         $values = array();
         foreach ($querydata as $w) {
-            $values = array_pad($values, count($values) + 4, $w);
+            $values = array_pad($values, count($values) + 5, $w);
         }
 
         $where = '
@@ -164,6 +169,8 @@ class PluginSearchInternal extends PluginSearch {
         if (isset($data['group'])) {
             $sql .= $groupjoin;
         }
+        $sql .= "
+                LEFT OUTER JOIN {usr_account_preference} h ON (u.id = h.usr AND h.field = 'hiderealname')";
 
         $sql .= $where . '
             ORDER BY ' . $data['orderby'];
@@ -192,6 +199,9 @@ class PluginSearchInternal extends PluginSearch {
         else {
             $options['orderby'] = 'u.firstname, u.lastname, u.id';
         }
+        if (isset($options['exclude'])) {
+            $options['exclude'] = intval($options['exclude']);
+        }
         return $options;
     }
     
@@ -215,7 +225,16 @@ class PluginSearchInternal extends PluginSearch {
 
     public static function admin_search_user($queries, $constraints, $offset, $limit, 
                                              $sortfield, $sortdir) {
-        $sort = $sortfield . ' ' . strtoupper($sortdir);
+        $sort = 'TRUE';
+        if (preg_match('/^[a-zA-Z_0-9"]+$/', $sortfield)) {
+            $sort = $sortfield;
+            if (strtoupper($sortdir) != 'DESC') {
+                $sort .= ' ASC';
+            }
+            else {
+                $sort .= ' DESC';
+            }
+        }
         $where = 'WHERE u.id <> 0 AND u.deleted = 0';
         $values = array();
 
@@ -228,6 +247,9 @@ class PluginSearchInternal extends PluginSearch {
             $where .= ' AND ( ';
             $str = array();
             foreach ($queries as $f) {
+                if (!preg_match('/^[a-zA-Z_0-9"]+$/', $f['field'])) {
+                    continue; // skip this field as it fails validation
+                }
                 $str[] = 'u.' . $f['field'] 
                     . PluginSearchInternal::match_expression($f['type'], $f['string'], $values, $ilike);
             }
@@ -280,7 +302,7 @@ class PluginSearchInternal extends PluginSearch {
         if ($count > 0) {
             $data = get_records_sql_assoc('
                 SELECT 
-                    u.id, u.firstname, u.lastname, u.username, u.email, u.staff,
+                    u.id, u.firstname, u.lastname, u.username, u.email, u.staff, u.profileicon,
                     u.active, NOT u.suspendedcusr IS NULL as suspended
                 FROM
                     {usr} u ' . $institutionsearch . $where . '
@@ -314,6 +336,7 @@ class PluginSearchInternal extends PluginSearch {
                     }
                 }
                 foreach ($data as &$item) {
+                    $item->username = display_username($item);
                     $item = (array)$item;
                 }
                 $data = array_values($data);
@@ -332,46 +355,95 @@ class PluginSearchInternal extends PluginSearch {
     }
 
 
-    public static function group_search_user($group, $queries, $constraints, $offset, $limit, $membershiptype) {
-        $where = 'WHERE gm.group = ?';
-        $values = array($group);
-
-        // Get the correct keyword for case insensitive LIKE
-        $ilike = db_ilike();
-
+    public static function group_search_user($group, $queries, $constraints, $offset, $limit, $membershiptype, $order=null) {
         // Only handle OR/AND expressions at the top level.  Eventually we may need subexpressions.
-
+        $searchsql = '';
+        $values = array();
         if (!empty($queries)) {
-            $where .= ' AND ( ';
+            $ilike = db_ilike();
+            $searchsql .= ' AND ( ';
             $str = array();
             foreach ($queries as $f) {
+                if (!preg_match('/^[a-zA-Z_0-9"]+$/', $f['field'])) {
+                    continue; // skip this field as it fails validation
+                }
                 $str[] = 'u.' . $f['field'] 
                     . PluginSearchInternal::match_expression($f['type'], $f['string'], $values, $ilike);
             }
-            $where .= join(' OR ', $str) . ') ';
+            $searchsql .= join(' OR ', $str) . ') ';
         }
 
-        $group_member = 'group_member';
-        if (!empty($membershiptype) && in_array($membershiptype, array('request', 'invite'))) {
-            $group_member .= '_' . $membershiptype;
-            $gm_role = '';
-            $gm_role_order = '';
+        if ($membershiptype == 'nonmember') {
+            $select = '
+                    u.id, u.firstname, u.lastname, u.username, u.email, u.profileicon, u.staff';
+            $from = '
+                FROM {usr} u
+                WHERE u.id > 0 AND u.deleted = 0 ' . $searchsql . '
+                    AND NOT u.id IN (SELECT member FROM {group_member} gm WHERE gm.group = ?)';
+            $values[] = $group;
+            $orderby = 'u.firstname, u.lastname, u.id';
         }
-        else {
-            $gm_role = ', gm.role';
-            $gm_role_order = "gm.role = 'admin' DESC, ";
+        else if ($membershiptype == 'notinvited') {
+            $select = '
+                    u.id, u.firstname, u.lastname, u.username, u.email, u.profileicon, u.staff';
+            $from = '
+                FROM {usr} u
+                WHERE u.id > 0 AND u.deleted = 0 ' . $searchsql . '
+                    AND NOT u.id IN (SELECT member FROM {group_member} gm WHERE gm.group = ?)
+                    AND NOT u.id IN (SELECT member FROM {group_member_invite} gmi WHERE gmi.group = ?)';
+            $values[] = $group;
+            $values[] = $group;
+            $orderby = 'u.firstname, u.lastname, u.id';
+        }
+        else if ($membershiptype == 'request') {
+            $select = '
+                    u.id, u.firstname, u.lastname, u.username, u.email, u.profileicon,
+                    u.staff, ' . db_format_tsfield('gmr.ctime', 'jointime');
+            $from = '
+                FROM {usr} u
+                    INNER JOIN {group_member_request} gmr ON (gmr.member = u.id)
+                WHERE u.id > 0 AND u.deleted = 0 ' . $searchsql . '
+                    AND gmr.group = ?';
+            $values[] = $group;
+            $orderby = 'u.firstname, u.lastname, gmr.ctime, u.id';
+        }
+        else if ($membershiptype == 'invite') {
+            $select = '
+                    u.id, u.firstname, u.lastname, u.username, u.email, u.profileicon,
+                    u.staff, ' . db_format_tsfield('gmi.ctime', 'jointime');
+            $from = '
+                FROM {usr} u
+                    INNER JOIN {group_member_invite} gmi ON (gmi.member = u.id)
+                WHERE u.id > 0 AND u.deleted = 0 ' . $searchsql . '
+                    AND gmi.group = ?';
+            $values[] = $group;
+            $orderby = 'u.firstname, u.lastname, gmi.ctime, u.id';
+        }
+        else { // All group members
+            $select = '
+                    u.id, u.firstname, u.lastname, u.username, u.email, u.profileicon,
+                    u.staff, ' . db_format_tsfield('gm.ctime', 'jointime') . ', gm.role';
+            $from = '
+                FROM {usr} u
+                    INNER JOIN {group_member} gm ON (gm.member = u.id)
+                WHERE u.id > 0 AND u.deleted = 0 ' . $searchsql . '
+                    AND gm.group = ?';
+            $values[] = $group;
+            $orderby = "gm.role = 'admin' DESC, u.firstname, u.lastname, gm.ctime, u.id";
+            if ($order == 'latest') {
+                $orderby = 'gm.ctime DESC, u.firstname, u.lastname, u.id';
+            }
         }
 
-        $count = get_field_sql('SELECT COUNT(*) FROM {usr} u INNER JOIN {' . $group_member . '} gm ON (gm.member = u.id) ' . $where, $values);
+        if ($order == 'random') {
+            $orderby = db_random();
+        }
+
+        $count = get_field_sql('SELECT COUNT(*)' . $from, $values);
 
         if ($count > 0) {
             $data = get_records_sql_assoc('
-                SELECT
-                    u.id, u.firstname, u.lastname, u.username, u.email, u.staff, ' . db_format_tsfield('gm.ctime', 'jointime') . $gm_role . '
-                FROM
-                    {usr} u
-                INNER JOIN {' . $group_member . '} gm ON (gm.member = u.id) ' . $where . '
-                ORDER BY ' . $gm_role_order . 'gm.ctime, u.firstname, u.lastname, u.id',
+                SELECT ' . $select . $from . ' ORDER BY ' . $orderby,
                 $values,
                 $offset,
                 $limit);
@@ -512,141 +584,44 @@ class PluginSearchInternal extends PluginSearch {
      *               ),
      *           );
      */
-    public static function search_group($query_string, $limit, $offset=0, $type='member') {
-        if (is_postgres()) {
-            return self::search_group_pg($query_string, $limit, $offset, $type);
-        } 
-        else if (is_mysql()) {
-            return self::search_group_my($query_string, $limit, $offset, $type);
-        } 
-        else {
-            throw new SQLException('search_group() is not implemented for your database engine (' . get_config('dbtype') . ')');
-        }
-    }
-
-    public static function search_group_pg($query_string, $limit, $offset, $type) {
+    public static function search_group($query_string, $limit, $offset=0, $type='member', $category='') {
         global $USER;
+        $data = array();
+
         $sql = "
-            SELECT
-                id, name, description, grouptype, jointype, ctime, mtime
             FROM
                 {group}
             WHERE (
-                name ILIKE '%' || ? || '%' 
-                OR description ILIKE '%' || ? || '%' 
-            ) AND deleted = ? ";
-        $values = array($query_string, $query_string, 0);
-        if ($type == 'member') {
-            $sql .=  'AND (
-                id IN (
-                    SELECT "group" FROM {group_member} WHERE member = ?
-                )
-            )';
-            $values[] = $USER->get('id');
+                name " . db_ilike() . " '%' || ? || '%'
+                OR description " . db_ilike() . " '%' || ? || '%'
+            ) AND deleted = 0 ";
+        $values = array($query_string, $query_string);
+
+        if (!$grouproles = join(',', array_keys($USER->get('grouproles')))) {
+            $grouproles = '-1';
+        }
+
+       if ($type == 'member') {
+            $sql .=  'AND id IN (' . $grouproles . ')';
         }
         else if ($type == 'notmember') {
-            $sql .=  'AND (
-                id NOT IN (
-                    SELECT "group" FROM {group_member} WHERE member = ?
-                )
-            )';
-            $values[] = $USER->get('id');
+            $sql .= 'AND id NOT IN (' . $grouproles . ')';
+        }
+        if (!empty($category)) {
+            if ($category == -1) { //find unassigned groups
+                $sql .= " AND category IS NULL";
+            } else {
+                $sql .= ' AND category = ?';
+                $values[] = $category;
+            }
         }
-        $sql .= 'ORDER BY name';
-        $data = get_records_sql_array($sql, $values, $offset, $limit);
 
-        $sql = "
-            SELECT
-                COUNT(*)
-            FROM
-                {group} u
-            WHERE (
-                name ILIKE '%' || ? || '%' 
-                OR description ILIKE '%' || ? || '%' 
-            ) AND deleted = ? ";
-        if ($type == 'member') {
-            $sql .= 'AND (
-                    id IN (
-                        SELECT "group" FROM {group_member} WHERE member = ?
-                    )
-                )
-            ';
-        }
-        else if ($type == 'notmember') {
-            $sql .= 'AND (
-                    id NOT IN (
-                        SELECT "group" FROM {group_member} WHERE member = ?
-                    )
-                )
-            ';
-        }
-        $count = get_field_sql($sql, $values);
+        $count = get_field_sql('SELECT COUNT(*) '.$sql, $values);
 
-        return array(
-            'count'   => $count,
-            'limit'   => $limit,
-            'offset'  => $offset,
-            'data'    => $data,
-        );
-    }
-
-    public static function search_group_my($query_string, $limit, $offset, $type) {
-        global $USER;
-        $sql = "
-            SELECT
-                id, name, description, grouptype, jointype, ctime, mtime
-            FROM
-                {group}
-            WHERE (
-                name LIKE '%' || ? || '%' 
-                OR description LIKE '%' || ? || '%' 
-            ) AND deleted = ? ";
-        $values = array($query_string, $query_string, 0);
-        if ($type == 'member') {
-            $sql .=  'AND (
-                id IN (
-                    SELECT gm.group FROM {group_member} gm WHERE gm.member = ?
-                )
-            )';
-            $values[] = $USER->get('id');
+        if ($count > 0) {
+            $sql = 'SELECT id, name, description, grouptype, jointype, public, ctime, mtime, category ' . $sql . ' ORDER BY name';
+            $data = get_records_sql_array($sql, $values, $offset, $limit);
         }
-        else if ($type == 'notmember') {
-            $sql .=  "AND (
-                id NOT IN (
-                    SELECT gm.group FROM {group_member} gm WHERE gm.member = ?
-                )
-            )";
-            $values[] = $USER->get('id');
-        }
-        $sql .= 'ORDER BY name';
-        $data = get_records_sql_array($sql, $values, $offset, $limit);
-
-        $sql = "
-            SELECT
-                COUNT(*)
-            FROM
-                {group} u
-            WHERE (
-                name LIKE '%' || ? || '%' 
-                OR description LIKE '%' || ? || '%' 
-            ) AND deleted = ? ";
-        if ($type == 'member') {
-            $sql .= 'AND (
-                    id IN (
-                        SELECT gm.group FROM {group_member} gm WHERE gm.member = ?
-                    )
-                )
-            ';
-        }
-        else if ($type == 'notmember') {
-            $sql .= "AND (
-                    id NOT IN (
-                        SELECT gm.group FROM {group_member} gm WHERE gm.member = ?
-                    )
-                )
-            ";
-        }
-        $count = get_field_sql($sql, $values);
 
         return array(
             'count'   => $count,
@@ -1171,5 +1146,3 @@ define('PREG_CLASS_PUNCTUATION',
  */
 define('PREG_CLASS_CJK', '\x{3041}-\x{30ff}\x{31f0}-\x{31ff}\x{3400}-\x{4db5}'.
 '\x{4e00}-\x{9fbb}\x{f900}-\x{fad9}');
-
-?>
